/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("psm.admin.ui.internal.editors.SystemArchitectContextEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("psm.admin.ui.internal.pages.SystemArchitectConfigurationFragment");
dojo.require("rmps.core.projects.ProjectClient");
dojo.require("psm.common.constants.sadmConstants");

dojo.require("jazz.ui.MessageBox");

var ProjectClient = rmps.core.projects.ProjectClient;

var DEFAULT_CONTEXT_PROPERTY = psm.common.constants.sadmConstants.DEFAULT_CONTEXT_PROPERTY;
var DEFAULT_WSNAME_PROPERTY = psm.common.constants.sadmConstants.DEFAULT_WSNAME_PROPERTY;
var DIRTY_GROUP_ID = "rmps.projectProperties.SystemArchitectContextEditor.dirty";

dojo.require("dojo.i18n");
dojo.requireLocalization("psm.admin.ui.internal.editors", "SystemArchitectContextEditorMessages");
var messages= dojo.i18n.getLocalization("psm.admin.ui.internal.editors", "SystemArchitectContextEditorMessages");

/**
 * Editor for viewing and setting the system architect context of a dm project.
 */
dojo.declare("psm.admin.ui.internal.editors.SystemArchitectContextEditor", [rmps.app.ui.internal.editors.BaseEditor], {

	isRefreshable: false, //override from superclass

	_context: null, //Uri; the currently selected context
	_wsName:null,
	_originalContext: null, //Uri; the original stored context 

	postCreate: function() {
		this.inherited(arguments);
		this._loadContext();
	},
	
	_loadContext: function() {
		this._context = null;
		this._originalContext = null;
		var getPropertyDeferred = ProjectClient.getProjectProperty(this.editorInput.projectConfigurationsInfo.projectId, DEFAULT_CONTEXT_PROPERTY);
		getPropertyDeferred.addCallback(this, function(propertyValue){
			//value is the URL of the SA context
			if (propertyValue.length > 0) {
				var def=ProjectClient.getProjectProperty(this.editorInput.projectConfigurationsInfo.projectId, DEFAULT_WSNAME_PROPERTY);
				  def.addCallback(this, function(wsNameValue){
					 this.contentWidget.setContext(propertyValue,wsNameValue);
					 this._context = propertyValue;
					 this._wsName=wsNameValue;
					 this._originalContext = propertyValue;  
				  });				
			} else {
				//treat it as property not set
				this.contentWidget.updateLabel(null,null);
			}
		});
		getPropertyDeferred.addErrback(this, function(error) {
			if (error.status === 404) {
				//property is not set; display appropriate message
				this.contentWidget.updateLabel(null,null);
			} else {
				this._generalErrorCallback(error);
			}
		});		
	},
	
	_generalErrorCallback: function(error) {
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: messages.generalErrorMessage,
			details: [{message: messages['generalErrorMessage.explanation'] + "<br>" + messages['generalErrorMessage.useraction']}]
		});
	},
	
	//overridden from superclass
	createContentWidget: function() {
		return new psm.admin.ui.internal.pages.SystemArchitectConfigurationFragment({
			contextSelected: dojo.hitch(this, this.contextSelected),
			projectConfigurationsInfo: this.getEditorInput().projectConfigurationsInfo,
			errorCallback: dojo.hitch(this, this._generalErrorCallback)
		});
	},
	
	/**
	 *Overridden from superclass.  Saves the value of the OSLC context.
	 */
	doSave: function(taskMonitor) {
		if (this._originalContext == this._context) {
			return;
		}
				
		var deferred = ProjectClient.setProjectProperty(this.editorInput.projectConfigurationsInfo.projectId, DEFAULT_CONTEXT_PROPERTY, this._context, true);
		deferred.addErrback(this, function(error) {
			if (error.status === 403) {
				return new Error(messages.error_save_permission);
			} else {
				return new Error(messages.generalErrorMessage + "\n" 
				                + messages["generalErrorMessage.explanation"] + "\n"
				                + messages["generalErrorMessage.useraction"]);
			}
		});
		deferred.addCallback(this, function(msg){
			ProjectClient.setProjectProperty(this.editorInput.projectConfigurationsInfo.projectId,DEFAULT_WSNAME_PROPERTY,this._wsName,false);
		});
		
		var task = taskMonitor.beginTask(messages.saveTaskLabel, 1);
		task.handleDeferredWork(deferred, 1);
		return deferred;
	},
	
	lightRefresh: function() {
		this._loadContext();
		this.removeDirty(this.contentWidget, DIRTY_GROUP_ID);
	},

	/**
	 * Called when a new context is selected.  If contextUrl is null, the existing context should be cleared.
	 */
	contextSelected: function(contextUrl,wsName) {
		this._context = contextUrl;
		this._wsName=wsName;
		if (this._context == this._originalContext) {
			this.removeDirty(this.contentWidget, DIRTY_GROUP_ID);
		} else {
			this.addDirty(this.contentWidget, DIRTY_GROUP_ID);
		}
	}

});

})();

