/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.rational.psm.importer.temp                                     |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corporation.          |
 *+------------------------------------------------------------------------+
 */

(function() {
	/**
	 * This widget provides the context section for SA import request
	 * 
	 * @author Xiangjun CHEN, Qi Shen, JunPing Cai
	 */
	dojo.provide("psm.admin.ui.internal.pages.ContextWidget");

	dojo.require("dojo.cache");
	dojo.require("dojo.i18n");
	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.requireLocalization("psm.admin.ui.internal.pages", "ContextWidgetMessages");
	dojo.requireLocalization("psm.importer.ui.internal.pages", "StandardMessages");
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");
	dojo.require("psm.common.utils.messaging");
	dojo.require("psm.common.constants.sadmConstants");
	
    var messages= dojo.i18n.getLocalization("psm.admin.ui.internal.pages","ContextWidgetMessages");
    var errorMessages = dojo.i18n.getLocalization("psm.importer.ui.internal.pages", "StandardMessages");
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var TRS_NAMESPACE = "http://jazz.net/ns/trs#";//$NON-NLS-1$ 
    var messagebox;
	var resourcelist = [];
	var uniqueDB = [];
	var uniqueEncy = [];

	dojo.declare("psm.admin.ui.internal.pages.ContextWidget", [ dijit._Widget, dijit._Templated ], {
		messages : messages,
		errorMessages : errorMessages,
		templateString : dojo.cache('psm.admin', 'ui/internal/pages/templates/ContextWidget.html'),//$NON-NLS-1$ //$NON-NLS-2$ 
        constructor:function(){
        	
        },
		postCreate : function() {
			this._evalProjectLinks();
			messagebox = new psm.common.utils.messaging();
			messagebox.initialiseMessageArea(this.contextMessageArea);//initial the error MessageArea
			dojo.connect(this.serverNameAttachPoint, "onchange", dojo.hitch(this, this._updateDBnames));//$NON-NLS-1$
			dojo.connect(this.DBnameAttachPoint, "onchange", dojo.hitch(this, this._updateEncynames));//$NON-NLS-1$
			dojo.connect(this.encyNameAttachPoint, "onchange", dojo.hitch(this, this._updateWsnames));//$NON-NLS-1$
			dojo.connect(this.wsNameAttachPoint, "onchange", dojo.hitch(this, this._changeWsnames));//$NON-NLS-1$
			
		},
		_cancel:function(){
			this.close();
		},

		_evalProjectLinks : function() {
			/*
			 * is called when the widget is loaded to access the dm/rootservices
			 */
			dojo.empty(this.serverNameAttachPoint);
			dojo.empty(this.DBnameAttachPoint);
			dojo.empty(this.encyNameAttachPoint);
			dojo.empty(this.wsNameAttachPoint);
			dojo.empty(this.previewText);
			this.DBnameAttachPoint.disabled=true;
			this.encyNameAttachPoint.disabled=true;
			this.wsNameAttachPoint.disabled=true;
			this.finishButton.disabled=true;

			var rootServicesUrl = ProcessConstants.ROOTSERVICESXML;
			var deferred = jazz.client.xhrGet({
				url : rootServicesUrl,
				headers : {
					"accept" : ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs : "xml", //$NON-NLS-1$
				load : dojo.hitch(this, this._determineLinkTypes),
				error : dojo.hitch(this, function(error) {
					return error;
				})
			});
		},

		_determineLinkTypes : function(rootServicesXml) {
			/*
			 * Callback function to handle the response(dm/rootservices)
			 */
			if (!rootServicesXml) {
				return;
			}

			var children = rootServicesXml.childNodes;
			var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var foundationRootServicesNodes = descriptionNode.childNodes;
			this._readServers(foundationRootServicesNodes);
		},

		_readServers : function(foundationRootServicesNodes) {
			/*
			 * Parse the dm/rootservices to get the friend element and its
			 * resource and access this resource using xhrGet
			 */
			var friendsNode = UIHelper.getNode(foundationRootServicesNodes, "friends", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
			var friendsResourceURL = UIHelper.getAttributeNodeValue(friendsNode,
					"resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var deferred = jazz.client.xhrGet({
				url : friendsResourceURL,
				headers : {
					"accept" : ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs : "xml", //$NON-NLS-1$
				load : dojo.hitch(this, this._populateServers),
				error : dojo.hitch(this, function(error) {
					
				})
			});
		},

		_populateServers : function(friendsListingXml) {
			/*
			 * Callback function to handle the dm/friend to get all the friend
			 * servers
			 */

			if (!friendsListingXml) {
				return ;
			}
			var children = friendsListingXml.childNodes;
			var rdfNode = UIHelper.getNode(children, "RDF", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var allFriends = UIHelper.getNodes(rdfNode.childNodes, "Friend", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$

			var servers = [];

			dojo.forEach(allFriends, function(friend) {
				var friendChildren = friend.childNodes;
				var titleNode = UIHelper.getNode(friendChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
				var friendTitle = titleNode.firstChild.data;
				friendTitle = decodeURIComponent(friendTitle);
				var rootServicesNode = UIHelper.getNode(friendChildren, "rootServices", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
				var rootServicesResourceNodeValue = UIHelper.getAttributeNodeValue(rootServicesNode,
						"resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				var applicationName = friendTitle + " ( " + rootServicesResourceNodeValue + " )";
				var anOption = new Option(applicationName, applicationName);
				anOption.resource = rootServicesResourceNodeValue;
				servers.push(anOption);
			}, this);

			var selectMessage = this.messages.selectApplication;
			selectOption = new Option(selectMessage, selectMessage);
			selectOption.resource = ProcessConstants.EMPTY;
			servers.unshift(selectOption);

			var optionArr = this.serverNameAttachPoint.options;
			dojo.forEach(servers, function(serverOption) {
				// appendChild(anOption) does not work on IE7,
				// so modify array directly
				optionArr[optionArr.length] = serverOption;
			}, this);
			return ;
		},

		_updateDBnames : function() {
			/* Get all DB instances for the selected server */
			dojo.empty(this.DBnameAttachPoint);
			dojo.empty(this.encyNameAttachPoint);
			dojo.empty(this.wsNameAttachPoint);
			dojo.empty(this.previewText);
			this.DBnameAttachPoint.disabled=true;
			this.encyNameAttachPoint.disabled=true;
			this.wsNameAttachPoint.disabled=true;
			this.finishButton.disabled=true;
			messagebox.hideMessage();
			var selectedServerUrl = this._getServerURL();
			if(selectedServerUrl=="")
				return;
            var deferred = jazz.client.xhrGet({
				url : selectedServerUrl,
				headers : {
					"accept" : ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs : "xml", //$NON-NLS-1$
				load : dojo.hitch(this, this._readRootServiceServiceProvider),
				error : dojo.hitch(this, function(error) {
					this.DBnameAttachPoint.disabled=true;
					messagebox.showMessage(this.errorMessages.admin_saServerOutOfService_summary_CRCSA10014E, jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							[{message:this.errorMessages.admin_saServerOutOfService_details_CRCSA10014E}], null, -1);
              		return error;
				})
			});

		},
		
		_getServerURL : function() {
			/* Get the Selected Server */
			if (this.serverNameAttachPoint.selectedIndex === -1 || this.serverNameAttachPoint.options.length === 0) {
				return ProcessConstants.EMPTY;
			}
			return this.serverNameAttachPoint.options[this.serverNameAttachPoint.selectedIndex].resource;
		},
		
		_readRootServiceServiceProvider:function(responseXml){
			var saUrl = this._getServerURL();
			var children = responseXml.childNodes;
			try{
                var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$ 
			    var serviceProvider = UIHelper.getNode(descriptionNode.childNodes,
					"amServiceProviders", ProcessConstants.NAMESPACE_OSLC_AM); //$NON-NLS-1$
			    var nodeValue = UIHelper.getAttributeNodeValue(serviceProvider, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
            }catch(e){
            	if(e instanceof TypeError){
            		this.DBnameAttachPoint.disabled=true;	
            		messagebox.showMessage(this.errorMessages.admin_severWrongSelected_summary_CRCSA10015E, jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							[{message:this.errorMessages.admin_severWrongSelected_details_CRCSA10015E}], null, -1);
              		return;
            	}
            }
            
            var deferred = jazz.client.xhrGet({
				url : nodeValue,
				headers : {
					"accept" : ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs : "xml", //$NON-NLS-1$
				load : dojo.hitch(this, this._readServiceProvidersResponseDM),
				error : dojo.hitch(this, function(error) {
					this.DBnameAttachPoint.disabled=true;
					if(error.toString() == "CancelError"){ //$NON-NLS-1$
						messagebox.showMessage(this.errorMessages.admin_authorisedToServerError_summary_CRCSA10013E, jazz.ui.MessageBox.STATUS_TYPES.ERROR, 
								[{message:this.errorMessages.admin_authorisedToServerError_details_CRCSA10013E}], null, -1);
					}else{
						messagebox.showMessage(this.errorMessages.admin_saServerOutOfService_summary_CRCSA10014E, jazz.ui.MessageBox.STATUS_TYPES.ERROR,
								[{message:this.errorMessages.admin_saServerOutOfService_details_CRCSA10014E}], null, -1);
					}
              		return error;
				})
			});
            
		},

		_readServiceProvidersResponseDM : function(serviceProviderXml) {
			/*
			 * Callback function to get all (DB,Ency,Workspace) pairs for the
			 * selected server
			 */
			resourcelist.splice(0, resourcelist.length);
			uniqueDB.splice(0, uniqueDB.length);
			uniqueEncy.splice(0, uniqueEncy.length);

			var children = serviceProviderXml.childNodes;
            try{
                var descriptionNode = UIHelper.getNode(children, "RDF", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$ 
			    var serviceProviders = UIHelper.getNode(descriptionNode.childNodes,
					"ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
			    var providerNodes = UIHelper.getNodes(serviceProviders.childNodes,
					"serviceProvider", ProcessConstants.NAMESPACE_OSLC_CORE); //$NON-NLS-1$
			    this.DBnameAttachPoint.disabled=false;
            }catch(e){
            	if(e instanceof TypeError){
            		this.DBnameAttachPoint.disabled=true;	
            		messagebox.showMessage(this.errorMessages.admin_severWrongSelected_summary_CRCSA10015E, jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							[{message:this.errorMessages.admin_severWrongSelected_details_CRCSA10015E}], null, -1);
              		return;
            	}
            }
            //Parse each node
            dojo.forEach(providerNodes, function(node) {
				var NodeValue = UIHelper.getAttributeNodeValue(node, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				var wsName=UIHelper.getAttributeNodeValue(node, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
				var NodeSeg = NodeValue.split("/");//$NON-NLS-1$
				var len = NodeSeg.length;
				var resource = {
					"res" : NodeValue, //$NON-NLS-1$
					"DB" : NodeSeg[len - 3], //$NON-NLS-1$
					"Ency" : NodeSeg[len - 2], //$NON-NLS-1$
					"Ws" : [ NodeSeg[len - 1], wsName]//$NON-NLS-1$
				};

				resourcelist.push(resource);
				// make sure the
				// DB,Ency,Workspace in the
				// array is unique
				if (dojo.indexOf(uniqueDB, NodeSeg[len - 3]) == -1) {
					uniqueDB.push(NodeSeg[len - 3]);
				}

				if (dojo.indexOf(uniqueEncy, NodeSeg[len - 2]) == -1) {
					uniqueEncy.push(NodeSeg[len - 2]);
				}

			}, this);

			var DBmessage = this.messages.selectDB;
			var Encymessage = this.messages.selectEncy;
			uniqueDB.unshift(DBmessage);
            uniqueEncy.unshift(Encymessage);

            this._populateUnqiueDB();

		},

		_populateUnqiueDB : function() {
			/* Display the DB in UI for the selected server */
			var sDBoptionArr = this.DBnameAttachPoint.options;
			dojo.forEach(uniqueDB, function(node) {
				var newOp = new Option(node, node);
				newOp.resource = node;
				sDBoptionArr[sDBoptionArr.length] = newOp;
			}, this);

		},

		_updateEncynames : function() {
			/* Display the Encys in UI for the selected DB */
			dojo.empty(this.encyNameAttachPoint);
			dojo.empty(this.wsNameAttachPoint);
			dojo.empty(this.previewText);
			this.encyNameAttachPoint.disabled=(this.DBnameAttachPoint.value == messages.selectDB);
			this.wsNameAttachPoint.disabled=true;
			this.finishButton.disabled=true;
			var sEncyoptionArr = this.encyNameAttachPoint.options;
			var selectedDB = this.DBnameAttachPoint.options[this.DBnameAttachPoint.selectedIndex].resource;
			dojo.forEach(uniqueEncy, function(node) {
				if (this._isBelongtoThisDB(node, selectedDB)) {
					var prasedNode = node.replace(new RegExp("%20","g")," ");
					var newOp = new Option(prasedNode, prasedNode);
					newOp.resource = node;
					sEncyoptionArr[sEncyoptionArr.length] = newOp;
				}
			}, this);
		},

		
		_updateWsnames : function() {
			/* Display the DB in UI for the selected DB and Ency */
			dojo.empty(this.wsNameAttachPoint);
			dojo.empty(this.previewText);
			this.wsNameAttachPoint.disabled=(this.encyNameAttachPoint.value == messages.selectEncy);
			this.finishButton.disabled=true;
			if(this.encyNameAttachPoint.value == messages.selectEncy)
				return;

			var sWsoptionArr = this.wsNameAttachPoint.options;
            var selectedDB = this.DBnameAttachPoint.options[this.DBnameAttachPoint.selectedIndex].resource;
			var selectedEncy = this.encyNameAttachPoint.options[this.encyNameAttachPoint.selectedIndex].resource;
			//Create a "Select Workspace.." option 
			var newOp = new Option(this.messages.wsMessage, this.messages.wsMessage);
			newOp.resource = this.messages.wsMessage;
			sWsoptionArr[sWsoptionArr.length] = newOp;
			
			dojo.forEach(resourcelist, function(resource) {
				var node=resource.Ws;
				if (resource.DB == selectedDB && resource.Ency == selectedEncy ){
					var newOp = new Option(node[1], node[1]);
					newOp.resource = node[0];
					sWsoptionArr[sWsoptionArr.length] = newOp;
				}
					

			}, this);
			
		},
		
		_changeWsnames: function(){
			var wsName = this.wsNameAttachPoint.options[this.wsNameAttachPoint.selectedIndex].resource;
			if(wsName!==this.messages.wsMessage){
			   this.finishButton.disabled=false;
			   dojo.empty(this.previewText);
			   this.previewText.appendChild(dojo.doc.createTextNode(this.getWsName()));
			}else{
			   dojo.empty(this.previewText);
			   this.finishButton.disabled=true;
			}
		},
		// get uri
		getDiagramTypeServerUri : function(){
			var serverName = this.serverNameAttachPoint.options[this.serverNameAttachPoint.selectedIndex].resource;
			var dbName = this.DBnameAttachPoint.options[this.DBnameAttachPoint.selectedIndex].resource;
			var encyName = this.encyNameAttachPoint.options[this.encyNameAttachPoint.selectedIndex].resource;
			var wsId = this.wsNameAttachPoint.options[this.wsNameAttachPoint.selectedIndex].resource;
			var serverUri = serverName.split("rootservices")[0]+"SQL/"+dbName+"/"+encyName+"/"+wsId;//$NON-NLS-1$ 
			return serverUri;
		},
		
		getWsName:function(){
			var serverName = this.serverNameAttachPoint.options[this.serverNameAttachPoint.selectedIndex].resource;
			var dbName = this.DBnameAttachPoint.options[this.DBnameAttachPoint.selectedIndex].resource;
			var fullName=this.wsNameAttachPoint.value;
			var hostname = (new dojo._Url(serverName)).host;
			return hostname + " ( " + dbName + " ) " + fullName;//$NON-NLS-1$ //$NON-NLS-2$ 
		},
		
		_isBelongtoThisDB : function(node, selectedDB) {

			var flag = false;
			dojo.forEach(resourcelist, function(resource) {
                if(node===this.messages.selectEncy&&selectedDB!== this.messages.selectDB)
                	flag = true;
				if (resource.DB == selectedDB && resource.Ency == node)
					flag = true;

			}, this);
			return flag;
		},

	});

})();