/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("psm.admin.ui.internal.pages.ProjectAreaEditorSadmContribution"); //$NON-NLS-1$
	dojo.require("dojo.cache");
	dojo.require("psm.admin.ui.internal.editors.SAContextEditorInput"); //$NON-NLS-1$
	dojo.require("psm.admin.ui.internal.editors.SystemArchitectContextEditor"); //$NON-NLS-1$
	dojo.require("rmps.core.TaskMonitor"); //$NON-NLS-1$
	dojo.require("rmps.app.ui.internal.util.DmProjectAreaEditorUtils"); //$NON-NLS-1$
	dojo.require("psm.admin.ui.internal.pages.ContextWidget");//$NON-NLS-1$
	dojo.require("jazz.ui.Dialog");//$NON-NLS-1$

	dojo.require("dojo.i18n");//$NON-NLS-1$
	dojo.requireLocalization("psm.admin.ui.internal.pages", "ProjectAreaEditorSadmContributionMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	var messages = dojo.i18n.getLocalization("psm.admin.ui.internal.pages", "ProjectAreaEditorSadmContributionMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	var DmProjectAreaEditorUtils = rmps.app.ui.internal.util.DmProjectAreaEditorUtils;
	
	dojo.declare("psm.admin.ui.internal.pages.ProjectAreaEditorSadmContribution", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$

		templateString : dojo.cache('psm.admin', 'ui/internal/pages/templates/ProjectAreaEditorSadmContribution.html'),//$NON-NLS-1$ //$NON-NLS-2$
		messages: messages,
		connectHandles:[],
		"class": "psm-admin-ui-internal-pages-ProjectAreaEditorSadmContribution",//$NON-NLS-1$ //$NON-NLS-2$
		
		editor: null,
		// tab only properties
		tabContainer: null,
		tabTitle: null,
		dialogWidget:null,
		saEditorInput:null,
		
		oldContext:null,
		oldWsName:null,
		
	    constructor: function(args) {
	    	this.editor = args.editor;
	    	this.tabContainer = args.tabContainer;
	    	this.tabTitle = args.tabTitle;
	    },
	    
		postCreate: function() {
			this.inherited(arguments);
			this._initSAContextSection();
			
		},

		_initSAContextSection: function() {
			this.saEditorInput = new psm.admin.ui.internal.editors.SAContextEditorInput(this.editor.projectConfigurationsInfo);
			this.saContextEditor = new psm.admin.ui.internal.editors.SystemArchitectContextEditor({
				suppressEditorUi: true,
				editorInput: this.saEditorInput,
				showLoading: function() {},
				hideMessage: function() {},
				changesPending: function() {
					return this.isDirty();
				},
				aboutToBeSaved: function(){},
				saved: function(){},
				dirtyStateChanged: dojo.hitch(this, function(isDirty) {
					this.editor.setDirty(this.editor.changesPending());
				})
			});
			this.saContextEditor.headerContainerNode.parentNode.removeChild(this.saContextEditor.headerContainerNode);
			this.saContextAttachPoint.appendChild(this.saContextEditor.domNode);
		},

		doSave: function(callback) {
			var changeContextDeferred= this._changeContext();
			changeContextDeferred.addCallback(this,function(propertyValue){
				this.save(callback);
			});
			changeContextDeferred.addErrback(this, function(propertyValue){
				this.tabContainer.displayErrorMessage(messages.errorSavingDmSettings);
		    });
	
        },
		
		_changeContext:function(){
			var projectId = this.saEditorInput.projectConfigurationsInfo.projectId;
			var query = {};
	        var uri = net.jazz.ajax._contextRoot + "/SADiagramContextChange";//$NON-NLS-1$
	        query.projectId=projectId;
	        var Deferred=jazz.client.xhrPost({
					url: uri+"?" + dojo.objectToQuery(query),//$NON-NLS-1$
					headers: {
						"Accept" : "text/xml",//$NON-NLS-1$ $NON-NLS-2$
						"Content-Type" : "text/html",//$NON-NLS-1$ $NON-NLS-2$
						"X-ibm-rmps-internal": "true" //$NON-NLS-1$ $NON-NLS-2$
					},
					content: {
					data: null
					}
		     });
	        return Deferred;
		},
		
		save: function(handleSuccess) {
			var taskMonitor = new rmps.core.TaskMonitor(messages.dmSave);
            var deferreds = [];
			var dl;
			var self = this;
			taskMonitor.setCanComplete(false);
			if (this.saContextEditor && this.saContextEditor.isDirty()) {
				oslcContextRequest = this.saContextEditor.doSave(taskMonitor);
				deferreds.push(oslcContextRequest);
			}
			taskMonitor.setCanComplete(true);
			if (deferreds.length > 0) {
				var dl = new dojo.DeferredList(deferreds);
				dl.then(dojo.hitch(this, function(results) {
					var errors = [];
					for (var i = 0; i < results.length; i++) {
						if (!results[i][0]) {
							if (results[i][1].message) {
								errors.push(results[i][1].message);
							} else if (typeof results[i][1] === 'string') { //$NON-NLS-1$
								errors.push(results[i][1]);
							} else if (Array.isArray(results[i][1])) {
								for (var j = 0; j < results[i][1].length; j++) {
									errors.push(results[i][1][j]);
								}
							} else {
								errors.push(messages.errorSavingDmSettings);
							}
						}
					}
					if (errors.length > 0) {
						// Show warnings for DM properties save errors
						this.editor.errObj = dojo.mixin({priority: ProcessConstants.ERROR_PRIORITY_LOW}, DmProjectAreaEditorUtils.handleErrors(this.tabTitle, errors));
					} else {
						/*
						 *  Refresh the tab if the DM contribution is hosted in a tab. 
						 *  JTS editor doesn't refresh tabs after the save automatically
						 */
						if (this.tabContainer) {
							this.refresh();
						}
					}
					if (handleSuccess && handleSuccess.call) {
						handleSuccess.call(this.editor, null);
					}
				}));
				return dl;
			}
		},
		
		/*
		 * Called from ProjectAreaEditor#customRefresh()
		 */
		refresh: function() {
			if (this.saContextEditor) {
				this.saContextEditor.lightRefresh();
			}
		},

		_clearSAContext: function() {
			this.oldContext = this.saContextEditor.contentWidget._contextLabel.firstChild.textContent; 
			this.oldWsName=this.saContextEditor.contentWidget._contextLabel.id;
			if (this.saContextEditor) {
				this.saContextEditor.contentWidget.clear();
				if(this.oldContext!==messages.contextNotSetLabel){
					this.editor._messageBox.showMessage({summary:messages.warningContextChangeMessage,closable: true,type:jazz.ui.MessageBox.STATUS_TYPES.WARNING});
			
				}
			}
		},
		
		_UpdateSAContext: function(url,wsName) {
			this.oldContext = this.saContextEditor.contentWidget._contextLabel.firstChild.textContent; 
			this.oldWsName=this.saContextEditor.contentWidget._contextLabel.id;
			if (this.saContextEditor) {
				this.saContextEditor.contentWidget.modify(url,wsName);
				if(this.oldContext!==messages.contextNotSetLabel&&this.oldContext!==url){
					this.editor._messageBox.showMessage({summary:messages.warningContextChangeMessage,closable: true,type:jazz.ui.MessageBox.STATUS_TYPES.WARNING})
				}
			}
		},
		/*
		 * Called When click the set button, it will pop up a SA Context Dialog
		 */
		_setSAContext: function() {
		
		  var addLink=new psm.admin.ui.internal.pages.ContextWidget();
		  var self=this;
		  var dialogWidget = new jazz.ui.Dialog({
			  primaryTitle: this.messages.saContext,
			  contentNode: addLink.domNode,
			  width:"auto",//$NON-NLS-1$
              destroyContents: true
			  });
			  dialogWidget.addLinkObj = addLink;
			  addLink.startup(); 
		 this.connectHandles.push(dojo.connect(addLink.cancelButton, "onclick", dialogWidget, function(e){//$NON-NLS-1$
				  dojo.stopEvent(e);
				  this.close();
				  }));
		 this.connectHandles.push(dojo.connect(addLink.finishButton, "onclick", dialogWidget, function(e){//$NON-NLS-1$
				  self._UpdateSAContext(this.addLinkObj.getDiagramTypeServerUri(),this.addLinkObj.getWsName());
				  this.close();
				  })); 
		},

		changesPending: function() {
			return (this.saContextEditor && this.saContextEditor.isDirty());
		},
		
		canSave: function() {
			return true;
		}
		
	});

})();
