/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.rational.psm.importer.temp                                     |
 *|                                                                        |
 *| Copyright IBM Corporation 2011-2013.  All Rights Reserved.             |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corporation.          |
 *+------------------------------------------------------------------------+
 */
(function() {
	dojo.provide("psm.importer.ui.internal.pages.SACommonUtils");
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	//for the RDF NAMESPACES 
	var RDFS_NAMESPACE="http://www.w3.org/2000/01/rdf-schema#";//$NON-NLS-1$
	var DCTERMS_NAMESPACE = "http://purl.org/dc/terms/";//$NON-NLS-1$
	var SA_NAMESPACE = "http://jazz.net/ns/sa#";//$NON-NLS-1$
	
	var saEntities = {
			"-spa-": " ",
			"-opb-": "(",
			"-que-": "?",
			"-per-": "%",
			"-clb-": ")",
			"-bws-": "\\",
			"-dol-": "$",
			"-dot-": ".",
			"-fws-": "/",
			"-amp-": "&",
			"-col-": ":",
			"-quo-": "'"
		};//$NON-NLS-1$
	
	psm.importer.ui.internal.pages.SACommonUtils = {
			/**
			 * to restore sa diagram type's name
			 * "str-spa-str" -> "str str"
			 */
			decodeSAName : function(sName) {
				var retval = sName;
				for( var ent in saEntities ) {
					retval = retval.replace( new RegExp(ent,"g") , saEntities[ent] );//$NON-NLS-1$
				}
				return retval;
			},
			
			/**
			 * opposite operation to _decodeSAName
			 * "str str" -> "str-spa-str"
			 */
			encodeSAName : function(sName) {
				var retval = sName;
				for( var ent in saEntities ) {				
					retval = retval.replace(new RegExp("\\"+saEntities[ent],"g"),ent);					
				}
				return retval;
			},
			
			/**
	    	 *  main method to parse from diagram type rdf file to list
	    	 */
			parseDiagramTypesOrInstances : function(result){  
				var listAll = [];
	    		var children = result.childNodes; 
	    		var Nodes_RDF = UIHelper.getNode(children,"RDF",ProcessConstants.NAMESPACE_RDF);//$NON-NLS-1$
	    		var Nodes_Description = UIHelper.getNode(Nodes_RDF.childNodes,"Description",ProcessConstants.NAMESPACE_RDF);//$NON-NLS-1$
	    		var members = UIHelper.getNodes(Nodes_Description.childNodes,"member",RDFS_NAMESPACE);//$NON-NLS-1$
				var len = members.length;
				dojo.forEach(members,dojo.hitch(this,function(member){
					var resources = UIHelper.getNode(member.childNodes,"resource",RDFS_NAMESPACE);//$NON-NLS-1$
					var title = UIHelper.getNode(resources.childNodes,"title",DCTERMS_NAMESPACE);//$NON-NLS-1$
					var diagramUri = UIHelper.getAttributeNodeValue(resources, "about", ProcessConstants.NAMESPACE_RDF);//$NON-NLS-1$
					var countNode=  UIHelper.getNode(resources.childNodes, "instanceCount", SA_NAMESPACE);//NON-NLS-1$ 
					var instanceCount = (typeof(countNode) == "undefined") ? "": countNode.childNodes[0].data;//$NON-NLS-1$ //$NON-NLS-2$;					
					var nameOfDiagram = this.decodeSAName(title.childNodes[0].data);
					var temp = {
							name:nameOfDiagram,
							instanceCount:instanceCount,
							uri:diagramUri
					}
					listAll.push(temp);
				}));
				var sortFunction = function(a,b){
					if(a.name>b.name){
						return 1;
					}
					return -1;			
				};
				return listAll.sort(sortFunction);	
	    	},
	    	
			
			
};


})();