/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	/**
	 * This widget provides the Diagram searching 
	 * 
	 * @author NanQun CHEN, XiangJun CHEN
	 */
	dojo.provide("psm.importer.ui.internal.pages.SAImportBySearching");
	
	dojo.require("jazz.ui.SearchBox");
	dojo.require("jazz.app.context");
	dojo.require("jazz.client.xhr");
	dojo.require("rmps.core.RootServices");
	dojo.require("psm.common.constants.sadmConstants");
	dojo.require("psm.importer.ui.internal.constants.iconConstants");
	dojo.require("rmps.core.projects.ProjectClient");
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");	
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");
	dojo.require("jazz.ui.ListBox");
	dojo.require("jazz.ui.ResourceLink");
	dojo.requireLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");

	var Constants = psm.common.constants.sadmConstants;
	var Context = jazz.app.context.get();
	var messages = dojo.i18n.getLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");   
	var sadmConstants = psm.common.constants.sadmConstants;
	var iconConstants = psm.importer.ui.internal.constants.iconConstants;
	var ProjectClient = rmps.core.projects.ProjectClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var ListBox = jazz.ui.ListBox;
	var resourceLink = jazz.ui.ResourceLink;
	dojo.declare("psm.importer.ui.internal.pages.SAImportBySearching",[ dijit._Widget, dijit._Templated ],{
		messages:messages,
		searchBox:null,
		server_uri:null,
		importActionButton:null,
		entries :[],
		instanceData:[],
		listBoxWidth: "353px",//$NON-NLS-1$
		listBoxHeight: "250px",//$NON-NLS-1$
		templateString :  
    		dojo.cache("psm.importer", "ui/internal/pages/templates/SAImportBySearching.html"),
		constructor:function(){
			this.server_uri = arguments[0].serverUri;
			this.importActionButton = arguments[0].button;
			this.searchBox = new jazz.ui.SearchBox({
				defaultText:messages.defaultSearchMessage,
				scopes:[{id:"all",label:messages.defaultSearchScope,icon:iconConstants.SEARCH_ICON}],//$NON-NLS-1$ 
				//onExecute: dojo.hitch(this,this._onExecute),
				onUpdate: dojo.hitch(this,this._onExecute),
				tooltip: messages.defaultSearchTip,
			});
		},
		postCreate : function() {
			this._selectedList = null;
			this._unselectedList = null;
			this.configureUI();
		},
		/**
		 * configure page UI
		 */
		configureUI: function() {
			var self = this;
			this._unselectedList = new ListBox({
				height: this.listBoxHeight,
				width: this.listBoxWidth,
				onClick: dojo.hitch(this,this._selectResults),
				ondblClick: function(entry) {
					self._handleEventDoubleClickedUnselected(entry);
				}
			});
			
			this._selectedList = new ListBox({
				height: this.listBoxHeight,
				width: this.listBoxWidth,
				onClick: dojo.hitch(this,this._selectInstances),
				ondblClick: function(entry) {
					self._handleEventDoubleClickedSelected(entry);
				}
			});
			this.searchResults.appendChild(this._unselectedList.domNode);
			this.selectedInstances.appendChild(this._selectedList.domNode);
			this.searchInput.appendChild(this.searchBox.domNode);
			this.addButtonImg.src = iconConstants.RIGHT_ARROW;
			this.removeButtonImg.src = iconConstants.LEFT_ARROW;
			dojo.connect(this.addButton,"onclick",dojo.hitch(this,this._addInstance));
			dojo.connect(this.removeButton,"onclick",dojo.hitch(this,this._removeInstance));
		},
		/**
		 *  update import button ,to disable it by no selection, else enable import button
		 */
		updateImportButton:function(){
			if(this._selectedList.getEntryCount()!=0){
				this.importActionButton.disabled=sadmConstants.EMPTYSTRING;
			}else{
				this.importActionButton.disabled="disabled";//$NON-NLS-1$
			}
		},
		
		/**
		 * load the searching results into diagram search box
		 */
		_loadData:function(){
			this._unselectedList.removeAllEntries();
			for(var i=0,len=this.instanceData.length;i<len;i++){
				this.entries[i]={
						"id":this.instanceData[i].id,//$NON-NLS-1$
						"value":this.instanceData[i].resource,//$NON-NLS-1$
						"label":this.instanceData[i].title,//$NON-NLS-1$
						"type":this.instanceData[i].type//$NON-NLS-1$
				};
				this._unselectedList.addEntry(this.entries[i]);
				this._addHover(this.entries[i],this._unselectedList);
				this._addSpan(this.entries[i],this._unselectedList);
				if(this._selectedList.getEntryCount()!=0){
					var selectedEntries = this._selectedList.getAllEntries();
					dojo.forEach(selectedEntries, dojo.hitch(this, function(entry,j) {
						if(this.entries[i].id==selectedEntries[j].id){
							var entryNode = this._unselectedList._getEntryNode(entry);
							for(var k in entryNode.children){
							dojo.addClass(entryNode.children[k], "selectedItems");//$NON-NLS-1$
							}
						}
					}));
				}
			}
		},
		
		/**
		 * double click action handler by the selection of diagram instances 
		 */
		_handleEventDoubleClickedUnselected:function(entry){
			var selectedEntry =this._selectedList.getEntry(entry.id);
			this._selectedList.addEntry(entry);
			var entryNode = this._unselectedList._getEntryNode(entry);
			if(selectedEntry==null){
				this._addHover(entry,this._selectedList);
				this._addSpan(entry,this._selectedList);
			}
			for(var i in entryNode.children){
				if(dojo.hasClass(entryNode.children[i],"selectedItems")){//$NON-NLS-1$
					dojo.removeClass(entryNode.children[i], "unselectedItems");//$NON-NLS-1$
				}
				dojo.addClass(entryNode.children[i], "selectedItems");//$NON-NLS-1$	
			}
			
			this.addButton.disabled="disabled";//$NON-NLS-1$
			this.removeButton.disabled="disabled";//$NON-NLS-1$
			this.updateImportButton();
		},
		
		/**
		 * double click action handler by the deselection of diagram instances 
		 */
		_handleEventDoubleClickedSelected:function(entry){
			this._selectedList.removeEntry(entry);
			if(this._unselectedList.getEntry(entry.id)!=null){
				var entryNode = this._unselectedList._getEntryNode(entry);
				for(var i in entryNode.children){
				if(dojo.hasClass( entryNode.children[i],"unselectedItems")){//$NON-NLS-1$
					dojo.removeClass(entryNode.children[i], "selectedItems");//$NON-NLS-1$
				}
				dojo.addClass(entryNode.children[i], "unselectedItems");//$NON-NLS
				}
			}
			this.addButton.disabled="disabled";//$NON-NLS-1$
			this.removeButton.disabled="disabled";//$NON-NLS-1$
			this.updateImportButton();
		},
		/**
		 * add button action
		 */
		_addInstance:function(){
			var entries = this._unselectedList.getSelection();
			if (entries.length === 0) {
				return;
			}else{
				for(var i = 0;i<entries.length;i++){
					var entryNode = this._unselectedList._getEntryNode(entries[i]);
					var selectedEntry =this._selectedList.getEntry(entries[i].id);
					for(var j in entryNode.children){
						if(dojo.hasClass(entryNode.children[j],"selectedItems")){//$NON-NLS-1$
							dojo.removeClass(entryNode.children[j], "unselectedItems");//$NON-NLS-1$
						}
						dojo.addClass(entryNode.children[j], "selectedItems");//$NON-NLS-1$
					}
					this._selectedList.addEntry(entries[i]);
					if(selectedEntry==null){
						this._addHover(entries[i],this._selectedList);
						this._addSpan(entries[i],this._selectedList);
					}
				}
			}
			this.addButton.disabled="disabled";//$NON-NLS-1$
			this.removeButton.disabled="disabled";//$NON-NLS-1$
			this.updateImportButton();
		},
		/**
		 * remove instance action
		 */
		_removeInstance:function(){
			var entries = this._selectedList.getSelection();
			// Validate entries
			if (entries.length === 0) {
				// NO-OP
				return;
			}else{
				for(var i =0;i<entries.length;i++){
					if(this._unselectedList.getEntry(entries[i].id)!=null){
						var entryNode = this._unselectedList._getEntryNode(entries[i]);
						for(var j in entryNode.children){
						if(dojo.hasClass(entryNode.children[j],"unselectedItems")){//$NON-NLS-1$
							dojo.removeClass(entryNode.children[j], "selectedItems");//$NON-NLS-1$
						}
						dojo.addClass(entryNode.children[j], "unselectedItems");//$NON-NLS
						}
					}
				}
			}
			this._selectedList.removeEntries(entries);
			this.addButton.disabled="disabled";//$NON-NLS-1$
			this.removeButton.disabled="disabled";//$NON-NLS-1$
			this.updateImportButton();
		},
		/**
		 * action handler by clicking of the search result box
		 */
		_selectResults:function(){
			this.addButton.disabled="";//$NON-NLS-1$
			this.removeButton.disabled="disabled";//$NON-NLS-1$
			this.updateImportButton();
		},
		
		/**
		 * action handler by clicking of the selected diagram box
		 */
		_selectInstances:function(){
			this.addButton.disabled="disabled";//$NON-NLS-1$
			this.removeButton.disabled="";//$NON-NLS-1$
			this.updateImportButton();
		},
		/**
		 * add diagram type for instances in list
		 */
		_addHover:function(entry,parent){
			var entryNode = parent._getEntryNode(entry);
			dojo.attr(entryNode, "href", entry.value);//$NON-NLS-1$
			var resLink = new resourceLink( { retainLinkText: true } , entryNode);
		},
		/**
		 * add column in listBox for diagram type
		 */
		_addSpan:function(entry,parent){
			var entryNode = parent._getEntryNode(entry);
			var typeNode = dojo.create("span",{ "class":"sa-import-request-search-listBox-type",innerHTML:"["+entry.type+"]"});//$NON-NLS-1$
			entryNode.appendChild(typeNode);
		},
		/**
		 *
		 * SA OSLC call to get searching result
		 */
		_onExecute:function(query){
    		var server_uri = this.server_uri + '/Instances?class=Diagram&filterbyname=' + query;//$NON-NLS-1$
			this.searchLoad.style.visibility="visible";//$NON-NLS-1$
			jazz.client.xhrGet({
				url:server_uri,	
	        	sync: false,
	        	handleAs: "xml",//$NON-NLS-1$
	        	load: dojo.hitch(this, function(result){
					this.searchLoad.style.visibility="hidden";//$NON-NLS-1$
					this._parseInstances(result);
					this._loadData();
	        	}),
	        	error: dojo.hitch(this, function(){
	        		this.searchLoad.style.visibility="hidden";//$NON-NLS-1$
	         	}),
	        	timeout: 5000,
	       });
		},
		
		/**
		 * parser SA OSLC call result into SA diagram instances list
		 */
		_parseInstances : function(result){  	
			this.instanceData=[];
    		var children = result.childNodes; 
    		var children1 = UIHelper.getNode(children,"Encyclopedia",null);//$NON-NLS-1$
    		var children2 = UIHelper.getNode(children1.childNodes,"Instances",null);//$NON-NLS-1$
    		var instances = UIHelper.getNodes(children2.childNodes,"Instance",null);//$NON-NLS-1$
    		for(var i=0,len=instances.length;i<len;i++){
				var title = instances[i].getAttribute("name").toString();//$NON-NLS-1$
				var type = instances[i].getAttribute("typename").toString();//$NON-NLS-1$
				var resource = instances[i].getAttribute("href");//$NON-NLS-1$
				var resourceArr = resource.split("/");//$NON-NLS-1$
				var id = resourceArr[resourceArr.length-1];
				var temp = {id:id,resource:resource,title:title,type:type};
				this.instanceData.push(temp);
    		}		
    	},
    	/**
    	 * Return the selected diagram instances for importing
    	 */
    	getSelectedDiagrams:function(){
    		var selectedDiagrams = [];
    		var entries = this._selectedList.getAllEntries();
			for(var i=0,len=entries.length;i<len;i++){
				var temp = {name:entries[i].label, uri:entries[i].value};
				selectedDiagrams.push(temp);
			}
			return selectedDiagrams;
    	},
	});
})();
