/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.rational.psm.importer.temp                                     |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corporation.          |
 *+------------------------------------------------------------------------+
 */

(function() {
	/**
	 * This widget provides the entry page for SA import history
	 * 
	 * @author Xiangjun CHEN, Qi SHEN, Jun Ping Cai
	 */
	dojo.provide("psm.importer.ui.internal.pages.SAImportHistory");

	dojo.require("dojo.cache");
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	dojo.require("dojo.i18n");
	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dojo.date.locale");
	dojo.require("jazz.app.context");
	dojo.require("jazz.ui.StyledTable");
	dojo.require("jazz.ui.SimpleToolbar");
	dojo.require("jazz.ui.PaginationControl");
	dojo.require("dojo.date.stamp");
	dojo.require("psm.common.constants.sadmConstants");
	dojo.require("psm.importer.ui.internal.constants.iconConstants");
	dojo.require("psm.common.utils.messaging");
	dojo.require("psm.importer.ui.internal.pages.hoverDialog");
	dojo.requireLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");
	dojo.requireLocalization("psm.importer.ui.internal.pages", "StandardMessages");
	
	var historyMessageBox = new psm.common.utils.messaging();
	var messages = dojo.i18n.getLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");
	var errorMessages = dojo.i18n.getLocalization("psm.importer.ui.internal.pages", "StandardMessages");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var sadmConstants = psm.common.constants.sadmConstants;
	var iconConstants = psm.importer.ui.internal.constants.iconConstants;
	/* Define the loading symbol image */
	var LOADING_IMG = "<img class=\"importLoadingImage\" title=\""+messages.loadingNow+"\" alt=\""+messages.loadingNow //$NON-NLS-1$ //$NON-NLS-2$
			+ "\" src="+iconConstants.LOADING_IMG+">"+messages.loadingNow; //$NON-NLS-1$
	
	/* Define the innerHTML segment for different action icons */
	var ACTION_DELETE = "<img class=\"imageCell\" title=\"" + messages.deleteButton + "\" alt=\"" + messages.deleteButton
			+ "\" src="+iconConstants.ACTION_DELETE+">";//$NON-NLS-1$
	var ACTION_CANCEL = "<img class=\"imageCell\" title=\"" + messages.cancelButton + "\" alt=\"" + messages.cancelButton
			+ "\" src="+iconConstants.ACTION_CANCEL+">";//$NON-NLS-1$
	var ACTION_DELETE_UNABLE = "<img class=\"imageCell\" title=\"" + messages.permissionDeniedButton + "\" alt=\"" + messages.permissionDeniedButton
		+ "\" src="+iconConstants.ACTION_DELETE_UNABLE+">";//$NON-NLS-1$
	var ACTION_CANCEL_UNABLE = "<img class=\"imageCell\" title=\"" + messages.permissionDeniedButton + "\" alt=\"" + messages.permissionDeniedButton
		+ "\" src="+iconConstants.ACTION_CANCEL_UNABLE+">";//$NON-NLS-1$

	/* Define the innerHTML segment for different Progress icons */
	var PROGRESS_COMPLETED = "<img class=\"imageCell\" title=\"" + messages.progress_completed + "\" alt=\""
			+ messages.progress_completed + "\" src="+iconConstants.PROGRESS_COMPLETED+">";//$NON-NLS-1$
	var PROGRESS_RUNNING = "<img class=\"imageCell\" title=\"" + messages.progress_running + "\" alt=\"" + messages.progress_running
			+ "\" src="+iconConstants.PROGRESS_RUNNING+">";//$NON-NLS-1$
	var PROGRESS_BROKEN = "<img class=\"imageCell\" title=\"" + messages.progress_failed + "\" alt=\"" + messages.progress_failed
			+ "\" src="+iconConstants.PROGRESS_BROKEN+">";//$NON-NLS-1$
	var PROGRESS_WAITING = "<img class=\"imageCell\"  title=\"" + messages.progress_waiting + "\" alt=\"" + messages.progress_waiting
			+ "\" src="+iconConstants.PROGRESS_WAITING+">";//$NON-NLS-1$
	var PROGRESS_CANCELED = "<img class=\"imageCell\" title=\"" + messages.progress_canceled + "\" alt=\"" + messages.progress_canceled
			+ "\" src="+iconConstants.PROGRESS_CANCELED+">";//$NON-NLS-1$
	var listAll = [];
	var logInfo = [];
	var args = {
		imgSrc : iconConstants.REFRESH_ICON,
		className : "jazz-ui-SimpleToolbar-button",
		title : messages.refresh,
		alt : messages.refresh,
		label : messages.refresh
	};
	var saEntities = {
			"-spa-": " ",//$NON-NLS-1$
			"-opb-": "(",//$NON-NLS-1$
			"-que-": "?",//$NON-NLS-1$
			"-per-": "%",//$NON-NLS-1$
			"-clb-": ")",//$NON-NLS-1$
			"-bws-": "\\",//$NON-NLS-1$
			"-dol-": "$",//$NON-NLS-1$
			"-dot-": ".",//$NON-NLS-1$
			"-fws-": "/",//$NON-NLS-1$
			"-amp-": "&",//$NON-NLS-1$
			"-col-": ":",//$NON-NLS-1$
			"-quo-": "'"//$NON-NLS-1$
		};
	
	dojo.declare("psm.importer.ui.internal.pages.SAImportHistory", [ dijit._Widget, dijit._Templated ], {
		messages : messages,
		errorMessages: errorMessages,
		templateString : '<div class="sa-import-historytable">' +
						'<div class="header" dojoAttachPoint="headerContainerNode">' +
						'<div dojoAttachPoint="historyMessageArea"></div>' +
						'<div class="toolbar" dojoAttachPoint="toolbarContainerNode">' +
						'</div>' +
						'<div class="mainContent" dojoAttachPoint="_headerContent">' +
						'<span class="title-icon"> <img dojoAttachPoint="_importStatusIMG" title=${messages.historyImg} alt=${messages.historyImg}>' +
						'</span>' + 
						'<span class="title">' + 
						'${messages.import_title}' +
						'</span>' +
						'</div>' +
						'</div>' +
						'<div dojoAttachPoint="pageControlArea"></div>' +
						'<div class="container" dojoAttachPoint="containerNode"></div>' +
						'<div dojoAttachPoint="loadingImageArea"></div>' +
						'</div>',//dojo.cache("psm.importer", "ui/internal/pages/templates/SAImportHistory.html"),//$NON-NLS-1$
		styledTable : null,
		simpleToolbar : null,
		rawInfo : null,
		interval : null,
		permissionChk : null,
		isFirstVisit:true,//when import history page created, set the isFirstVisit to true
		currentPage:1,
		sumPage:null,
		pageSize:sadmConstants.SA_IMPORT_HISTORY_PAGESIZE,

		postCreate : function() {

			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction("psm.importer.sa.history", this, "_checkPermission");//$NON-NLS-1$
			this._importStatusIMG.src = iconConstants.IMPORTSTATUS_ICON;
			this.styledTable = new jazz.ui.StyledTable({
				columns : [ {
					label : "", width : "2%"},//$NON-NLS-1$
				{
					label : this.messages.column_progress,
					width : "7%", sortable : false},//$NON-NLS-1$
				{
					label : this.messages.column_requester,
					width : "7%", sortable : false},//$NON-NLS-1$
				{
					label : this.messages.column_requesttime,
					width : "18%", sortable : false},//$NON-NLS-1$
				{
					label : this.messages.column_starttime,
					width : "18%", sortable : false},//$NON-NLS-1$
				{
					label : this.messages.column_duration,
					width : "15%", sortable : false},//$NON-NLS-1$
				{
					label : this.messages.column_stage,
					width : "23%", sortable : false},//$NON-NLS-1$
				{
					label : this.messages.column_action,
					width : "5%", sortable : false}],//$NON-NLS-1$
				showHover : true,
				sortByAscending : false,
				style : "width:100%",//$NON-NLS-1$				                                     
			});
			this.loadingImageArea.appendChild(dojo.create("div",{innerHTML:LOADING_IMG}));
			this.containerNode.appendChild(this.styledTable.domNode);
			this.simpleToolbar = new jazz.ui.SimpleToolbar();
			this.toolbarContainerNode.appendChild(this.simpleToolbar.domNode);
			this.simpleToolbar.addButton(this, this._refresh, args);
			this.rawInfo = [];
			historyMessageBox.initialiseMessageArea(this.historyMessageArea);
			this._updateTimer = setInterval(dojo.hitch(this, this._autoUpdate), sadmConstants.AUTO_REFRESH_TIME);
		},
		
		destroy: function(){
			if(this.updateTimer){
				clearInterval(this._updateTimer);
				delete this._updateTimer;
			}
		},
		
		_autoUpdate:function(){
			var query = {};
			var context = jazz.app.context.get();
			var uri = net.jazz.ajax._contextRoot + "/ImportHistoryUpdate";//$NON-NLS-1$
			query.projectId = context.itemId;
		    query.requestIds = this._filterRequestIDs();
		    if(query.requestIds!=sadmConstants.EMPTYSTRING){
		    	jazz.client.xhrGet({
		    		url : uri + "?" + dojo.objectToQuery(query),//$NON-NLS-1$
		    		headers : {
		    			"Accept" : "text/xml",//$NON-NLS-1$ $NON-NLS-2$
		    			"Content-Type" : "text/html",//$NON-NLS-1$ $NON-NLS-2$
		    			"X-ibm-rmps-internal" : "true" //$NON-NLS-1$ $NON-NLS-2$
		    		},
		    		content : {
		    			data : null
		    		},

		    		load : dojo.hitch(this, this._updateByRaws),
		    		error : dojo.hitch(this, function(result) {
		    		})
		    	});
		    }
		},	
		
		_filterRequestIDs:function(){
			var requestIds=sadmConstants.EMPTYSTRING;
			dojo.forEach(this.rawInfo,function(raw){
				var status=raw.progress;
				if(status==sadmConstants.SA_IMPORT_PROGRESS_WAITING||status==sadmConstants.SA_IMPORT_PROGRESS_RUNNING||status==sadmConstants.SA_IMPORT_PROGRESS_CANCELING){
				   requestIds +=raw.key+','; 
				}
			},this);
			
			if(requestIds==sadmConstants.EMPTYSTRING){
				clearInterval(this._updateTimer);
				delete this._updateTimer;
			}
			
			return requestIds.slice(0,requestIds.length-1);
		},
		
		_updateByRaws:function(jsonString){
			var jsonObject = eval("(" + jsonString + ")");
			var raws=this._parseJSON(jsonObject);
			this._updateRawInfo(raws);
			dojo.forEach(raws, function(raw) {
				this._updateByRaw(raw);
			}, this);
		},
		
		_updateByRaw:function(raw){
			var currentStatus=raw.progress;
			if(currentStatus!=messages.progress_waiting){
				var tr=dojo.byId(raw.key);
				var table = tr.parentNode;
				table.replaceChild(this._createTr(raw),tr);

			}
		},
		
		_updateRawInfo:function(raws){
			dojo.forEach(raws, function(raw) {
				var importRequestID=raw.key;
				for ( var i = 0; i < this.rawInfo.length; i++) {
					if(this.rawInfo[i].key == importRequestID){
						this.rawInfo.splice(i,1);
						this.rawInfo.push(raw);
						return;
					}
				}
			}, this);
		},

		/**
		 * Create the tr for import history table according to the given
		 * parameters
		 */
		_createTr : function(raw) {

			var tr = dojo.create("tr", {
				id : raw.key,
			});
			var icon = this._getProgressIcon(raw.progress);
			// get the Local time from the UTC ISOString
			var localRequestTime = this._getLocalTime(raw.requestTime);
			var localStartTime = this._getLocalTime(raw.startTime);
			var td1 = dojo.create("td", {
				innerHTML : icon,
			});
			var progressString = "";
			if (raw.progress == sadmConstants.SA_IMPORT_PROGRESS_COMPLETED){
				progressString = messages.progress_completed;
			} else if (raw.progress == sadmConstants.SA_IMPORT_PROGRESS_FAILED) {
				progressString = messages.progress_failed;
			} else if (raw.progress == sadmConstants.SA_IMPORT_PROGRESS_CANCELED) {
				progressString = messages.progress_canceled;
			} else if (raw.progress == sadmConstants.SA_IMPORT_PROGRESS_WAITING) {
				progressString = messages.progress_waiting;
			} else if (raw.progress == sadmConstants.SA_IMPORT_PROGRESS_RUNNING) {
				progressString = messages.progress_running;
			} else if (raw.progress == sadmConstants.SA_IMPORT_PROGRESS_CANCELING) {
				progressString = messages.progress_cancel_accepted;
			}
			var td2 = dojo.create("td",{//$NON-NLS-1$
				innerHTML : "<span class='importHistoryRepeatStyle'><label>"//$NON-NLS-1$
				+ progressString + "</label></span>",//$NON-NLS-1$
			});

			var td3 = dojo.create("td", {
				innerHTML : raw.requester,
			});
			var td4 = dojo.create("td", {
				innerHTML : localRequestTime,
			});
			var td5 = dojo.create("td", {
				innerHTML : localStartTime,
			});
			var durationTime;
			if (raw.duration =="-") {
				durationTime = raw.duration;
			} else {
				durationTime = this._getDuration(raw.duration)
			}
			var td6 = dojo.create("td", {
				innerHTML : durationTime,
			});
			
			var stageString = "";
			if (raw.stage == sadmConstants.SA_IMPORT_STAGE_START){
				stageString = messages.ImportHistory_Stage_GettingDiagram_Msg;
			} else if (raw.stage == sadmConstants.SA_IMPORT_STAGE_CREATING_MODEL) {
				stageString = messages.ImportHistory_Stage_CreatingModel_Msg;
			} else if (raw.stage == sadmConstants.SA_IMPORT_STAGE_STORING_CACHE) {
				stageString = messages.ImportHistory_Stage_storingCache_Msg;
			} else if (raw.stage == sadmConstants.SA_IMPORT_STAGE_SUCCESS) {
				stageString = messages.ImportHistory_Stage_Successfully_Msg;
			} else if (raw.stage == sadmConstants.SA_IMPORT_STAGE_CANCELED) {
				stageString = messages.ImportHistory_Stage_canceled_Msg;
			} else if (raw.stage == sadmConstants.SA_IMPORT_STAGE_BROKEN) {
				stageString = messages.ImportHistory_Stage_broken_Msg;
			} else if (raw.stage == sadmConstants.SA_IMPORT_STAGE_WAITINGREBOOT) {
				stageString = messages.ImportHistory_Stage_canceled_Msg;
			} else if (raw.stage == sadmConstants.SA_IMPORT_STAGE_RUNNINGREBOOT) {
				stageString = messages.ImportHistory_Stage_broken_Msg;
			}else{
				var infos=raw.stage.split(' '); //$NON-NLS-1$
				if(infos.length > 1){
					if(infos[0] == sadmConstants.SA_IMPORT_STAGE_CachingDiagram){
						stageString = dojo.string.substitute(
								messages.ImportHistory_Stage_CachingDiagram_Msg,
								{processedDiagramNumber:infos[1], diagramNumbers: infos[3]});
					}else{
						stageString = '-';
					}
				}else{
					stageString = '-';
				}
			}
			var td7 = dojo.create("td", {
				innerHTML : "<span class='importHistoryRepeatStyle'>"//$NON-NLS-1$
					+ stageString + "</span>",
			});
			
			var td8 = null;
			if (raw.progress == sadmConstants.SA_IMPORT_PROGRESS_COMPLETED
					|| raw.progress == sadmConstants.SA_IMPORT_PROGRESS_FAILED
					|| raw.progress == sadmConstants.SA_IMPORT_PROGRESS_CANCELED) {
				if(this.permissionChk == sadmConstants.SA_PERMISSIONACCESS){
					td8 = dojo.create("td", {
						innerHTML : ACTION_DELETE,
					});
					dojo.connect(td8.firstChild, "onclick", { //$NON-NLS-1$
						selectedRow : tr,
						rawInfo:this.rawInfo,
						scopeOfThisClass:this
					}, this._doDelete);
				}else{
					td8 = dojo.create("td", {
						innerHTML : ACTION_DELETE_UNABLE,
					});
				}
			}else if (raw.progress == sadmConstants.SA_IMPORT_PROGRESS_WAITING
					|| raw.progress == sadmConstants.SA_IMPORT_PROGRESS_RUNNING) {
				if(this.permissionChk == sadmConstants.SA_PERMISSIONACCESS){
					td8 = dojo.create("td", {
						innerHTML : ACTION_CANCEL,
					});
					dojo.connect(td8.firstChild, "onclick", { //$NON-NLS-1$
							selectedRow : tr,
							message : this.messages.progress_canceled,
							container: this
						}, this._doCancel);

				}else{
					td8 = dojo.create("td", {
						innerHTML : ACTION_CANCEL_UNABLE,
					});
				}
			}else {
				td8 = dojo.create("td");
			}
			this._createResourceHoverInfo(td2.firstChild,raw);
			if(raw.stage != "-")
				this._createLogHoverInfo(td7.firstChild,raw);
			
			tr.appendChild(td1);
			tr.appendChild(td2);
			tr.appendChild(td3);
			tr.appendChild(td4);
			tr.appendChild(td5);
			tr.appendChild(td6);
			tr.appendChild(td7);
			tr.appendChild(td8);
			return tr;

		},
	/**
	* handle the planned time
	*/
	_handlePlannedTime:function(t1,t2){
		time = this._convertTime(t1);
		delayTime = parseInt(t2) * 1000;
		planTime = time.getTime() + delayTime;
		return planTime;	
      },						 				 
	/**
	 * create the hover info for import request record
	*/
	_createResourceHoverInfo : function(dom,raw) {
		var args={
			around:dom,
			icon:"<img  src="+iconConstants.IMPORTSTATUS_ICON+">",//$NON-NLS-1$
			title:messages.recordTitle,
			className:"sa-import-history-hover"//$NON-NLS-1$
		};
		var hoverUI = new psm.importer.ui.internal.pages.hoverDialog(args);
		var data = {};
		//connection info
		var saConnection = dojo.create("div",{innerHTML:raw.saWorkspace});//$NON-NLS-1$
		data[messages.connectionInformation]= this._generateOutHtml(saConnection);
		//diagram selection types
		var diagramTypes=this._getDiagramNameList(raw);
		if(diagramTypes!=sadmConstants.EMPTYSTRING)
			data[messages.historySelectionDiagramType] = diagramTypes;
		//diagram instances
		var diagramInstances=this._getDiagramInstances(raw);
		if(diagramInstances!=sadmConstants.EMPTYSTRING&&raw.importType != sadmConstants.SA_IMPORT_HISTORY_IMPORT_ALL)
			data[messages.historySelectionDiagramInstance] =diagramInstances;
		//schedule info
		var schedule = this._getScheduleInfo(raw);
		data[messages.historySchedule] = schedule.detail;
		//last import & next import
		if(raw.scheduleMode != sadmConstants.SA_IMPORT_HISTORY_SCHEDULE_NO_REPEAT){
			data[messages.lastImport] = schedule.lastTimeMessage;
			if (raw.progress != sadmConstants.SA_IMPORT_PROGRESS_FAILED) {
				data[messages.nextScheduled] = schedule.nextDate;
			}
		}
		//target workspace
		data[messages.streamName] = raw.streamName;
		//snapshot
		if(raw.snapshotName != "-"){
			raw.snapshotUrl = this._getSADMServerURI()+"dm/web/projects/"+jazz.app.context.get().name+"?rmps.context="+raw.snapshotUrl+"&oslc_config.context="+raw.snapshotUrl;//$NON-NLS-1$//$NON-NLS-2$
			data[messages.snapshotNameInHistory] = "<a href=\""+raw.snapshotUrl+"\" target=\"_blank\">"+raw.snapshotName+"</a>";//$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$
		}			
		hoverUI.addLine(data);	
	},
		/**
		 * create log info page
		 */
	    _createLogHoverInfo : function(dom,raw) {// handle the rowInfo resource by selectId
	    	var args = {
	    		around:dom,
	    		icon:this._getProgressIcon(raw.progress),
	    		title:messages.importLogTitle + " " + this._praseEncyName(raw.saContext),
	    		className:"sa-import-history-hover"//$NON-NLS-1$
	    	};
	    	var hoverUI = new psm.importer.ui.internal.pages.hoverDialog(args);
			var data={},details={},errors={},reasons={};
			var infos=raw.fullLog.split(',');//$NON-NLS-1$
            if(infos.length>1){
    			for (var i = 0; i < infos.length; i++) {
    				var myRegexp = /(\D*)(\d*)/g;//$NON-NLS-1$
    				var match = myRegexp.exec(infos[i]);
    				match[1] = match[1].split(':')[0]; //$NON-NLS-1$
    				if(match[1] == sadmConstants.SA_IMPORT_HISTORY_LOG_PROCESSED_DIAGRAM_NUMBER)
    					details[messages.ImportHistory_log_processedDiagramNumber_Msg] = match[2];
    				else{
    					match[1] = match[1].split(' ')[1]; //$NON-NLS-1$
    					if(match[1] == sadmConstants.SA_IMPORT_HISTORY_LOG_ADDED_DIAGRAM_NUMBER)
    						details[messages.ImportHistory_log_addedDiagramNumber_Msg] = match[2];
    					else if(match[1] == sadmConstants.SA_IMPORT_HISTORY_LOG_UPDATED_DIAGRAM_NUMBER)
    						details[messages.ImportHistory_log_updatedDiagramNumber_Msg] = match[2];
    					else if(match[1] == sadmConstants.SA_IMPORT_HISTORY_LOG_SKIPPED_DIAGRAM_NUMBER)
    						details[messages.ImportHistory_log_skippedDiagramNumber_Msg] = match[2];
    					else if(match[1] == sadmConstants.SA_IMPORT_HISTORY_LOG_FAILED_DIAGRAM_NUMBER)
    						details[messages.ImportHistory_log_failedDiagramNumber_Msg] = match[2];
    				}
    			}
			}else{
				details[infos[0]]='';
			}
			if(Object.keys(details).length>0){
				data[messages.stageHoverThDetails] = details;
			}
			if(raw.executionException){
				if(raw.executionException == sadmConstants.SADMEXCEPTION_HISTORY_EXECUTION)
					errors[messages.SADMException_Execution_Msg] = null;
				if(raw.executionException == sadmConstants.SADMEXCEPTION_HISTORY_FAILEDCONNECTSERVER)
					errors[messages.SADMException_Execution_FailedConnectServer_Msg] = null;
				if(raw.executionException == sadmConstants.SADMEXCEPTION_HISTORY_LINKTOSABROKEN)
					errors[messages.SADMException_Execution_LinkToSAServerBroken_Msg] = null;
				if(raw.executionException == sadmConstants.SADMEXCEPTION_HISTORY_SADMSHUTDOWN)
					errors[messages.SADMException_Execution_SADMShutDown_Msg] = null;
				if(raw.executionException == sadmConstants.SADMEXCEPTION_HISTORY_SNAPSHOT)
					errors[messages.SADMException_Execution_SnapShotAlreadyExist_Msg] = null;
				if(raw.executionException == sadmConstants.SADMEXCEPTION_HISTORY_SAPERMISSIONDENIED)
					errors[messages.SADMException_Execution_SAPermissionDenied_Msg] = null;
				if(raw.executionException == sadmConstants.SADMEXCEPTION_HISTORY_DIAGRAMMODEL)
					errors[messages.SADMException_Execution_DiagramModel_Msg] = null;
				if(raw.executionException == sadmConstants.SADMEXCEPTION_HISTORY_RUNTIME)
					errors[messages.SADMException_Execution_Runtime_Msg] = null;
				else{
					var infos=raw.executionException.split(':'); //$NON-NLS-1$
					if(infos[0] ==sadmConstants.SADMEXCEPTION_HISTORY_INTERNAL){
						errors[messages.SADMException_Execution_Interal_Msg] = infos[1];
					}
				}
				data[messages.stageHoverThErrors] = errors;
			}
			if(raw.cancelInfo){
				if(raw.cancelInfo == sadmConstants.SA_IMPORT_HISTORY_MANUAL_CANCEL_INFO)
					reasons[messages.ImportHistory_manualCancelInfo_Msg] = null;
				else if(raw.cancelInfo == sadmConstants.SA_IMPORT_HISTORY_RESET_CANCEL_INFO)
					reasons[messages.ImportHistory_resetCancelInfo_Msg] = null;
				else if(raw.cancelInfo == sadmConstants.SA_IMPORT_HISTORY_REBOOT_CANCEL_INFO)
					reasons[messages.ImportHistory_CancelInfo_WaitingReboot_Msg] = null;
				data[messages.stageHoverThCancelInfo] = reasons;
			}
			hoverUI.addLine(data);
    	},
		/**
		 * Delete the selected import history record
		 */
		_doDelete : function() {
			var importRequestID = this.selectedRow.id;
			var query = {};
			var context = jazz.app.context.get();
			var uri = net.jazz.ajax._contextRoot + "/ImportHistory";//$NON-NLS-1$
			query.projectId = context.itemId;
			query.requestId = importRequestID;
			jazz.client.xhrDelete({
				url : uri + "?" + dojo.objectToQuery(query),//$NON-NLS-1$				
				headers : {
					"Accept" : "text/xml",//$NON-NLS-1$ $NON-NLS-2$
					"Content-Type" : "text/html",//$NON-NLS-1$ $NON-NLS-2$
					"X-ibm-rmps-internal" : "true" //$NON-NLS-1$ $NON-NLS-2$
				},
				content : {
					data : null
				},

				load : dojo.hitch(this, function(result) {
					if(result.indexOf(sadmConstants.SA_PERMISSIONDENIED) >= 0){
						historyMessageBox.showMessage(errorMessages.history_cancelOrdeletePermissionDenied_summary_CRCSA10009E,jazz.ui.MessageBox.STATUS_TYPES.ERROR,
								[{message: errorMessages.history_cancel_deletePermissionDenied_details_CRCSA10009E}],null,-1);
					}else{
						var table = this.selectedRow.parentNode;
						table.removeChild(this.selectedRow);
						for ( var i = 0; i < this.rawInfo.length; i++) {
							if(this.rawInfo[i].key == importRequestID)
								this.rawInfo.splice(i,1);
						}
						this.scopeOfThisClass._refresh();
					}
				}),
				error : dojo.hitch(this, function(result) {
					if (result.responseText.match(sadmConstants.SA_IMPORT_REQUEST_NOT_EXISTED_MSG)){
						historyMessageBox.showMessage(errorMessages.history_illegalDeleteStateExecption_summary_CRCSA10008W, jazz.ui.MessageBox.STATUS_TYPES.WARNING,
								[{message: errorMessages.history_illegalDeleteStateExecption_details_CRCSA10008W}]);
					}else if(result.message.match(sadmConstants.SA_IMPORT_HISTORY_TIME_OUT_MSG)){
						historyMessageBox.showMessage(errorMessages.history_timeoutForDeleteTask_CRCSA10012E, jazz.ui.MessageBox.STATUS_TYPES.WARNING,
								[{message: errorMessages.history_timeoutForDeleteTask_details_CRCSA10012E}]);
					}else{
						historyMessageBox.showMessage(result.responseText, jazz.ui.MessageBox.STATUS_TYPES.ERROR);
					}
					
				})
			});
		},

		/**
		 * Cancel the selected import history record
		 */
		_doCancel : function() {
			var query = {};
			var uri = net.jazz.ajax._contextRoot + "/SADiagramImportCancel";//$NON-NLS-1$
			query.projectId = jazz.app.context.get().itemId;
			query.requestId = this.selectedRow.id;
			jazz.client.xhrPost({
				url : uri + "?" + dojo.objectToQuery(query),//$NON-NLS-1$
				headers : {
					"Accept" : "text/xml",//$NON-NLS-1$ $NON-NLS-2$
					"Content-Type" : "text/html",//$NON-NLS-1$ $NON-NLS-2$
					"X-ibm-rmps-internal" : "true" //$NON-NLS-1$ $NON-NLS-2$
				},
				content : {
					data : null
				},
				load : dojo.hitch(this, function(result) {
					if(result.indexOf(sadmConstants.SA_PERMISSIONDENIED) > 0){
						historyMessageBox.showMessage(errorMessages.history_cancelOrdeletePermissionDenied_summary_CRCSA10009E, jazz.ui.MessageBox.STATUS_TYPES.ERROR,
								[{message: errorMessages.history_cancel_deletePermissionDenied_details_CRCSA10009E}],null,-1);
					}else if(result){
						var cells = this.selectedRow.cells;
						var textNode = dojo.create("label",{ //$NON-NLS-1$
							innerHTML:messages.progress_cancel_accepted
						});
						var icon = dojo.create("img");//$NON-NLS-1$
						icon.setAttribute("class","imageCell");//$NON-NLS-1$ $NON-NLS-2$
						icon.setAttribute("title",messages.runningImg);//$NON-NLS-1$
						icon.setAttribute("alt",messages.runningImg);//$NON-NLS-1$
						icon.setAttribute("src", iconConstants.PROGRESS_RUNNING);//$NON-NLS-1$
						cells[0].replaceChild(icon,cells[0].firstChild);
						cells[1].firstChild.replaceChild(textNode,cells[1].firstChild.firstChild);
						cells[7].removeChild(cells[7].firstChild);
					}else{
						return;
					}
				}),
				error : dojo.hitch(this, function(result){
					if (result.responseText.match(sadmConstants.SA_IMPORT_REQUEST_NOT_EXISTED_MSG)){
						historyMessageBox.showMessage(errorMessages.history_illegalCancelStateExecption_summary_CRCSA10007W, jazz.ui.MessageBox.STATUS_TYPES.WARNING,
								[{message: errorMessages.history_illegalCancelStateExecption_details_CRCSA10007W}]);
					} else {
						historyMessageBox.showMessage(result.responseText, jazz.ui.MessageBox.STATUS_TYPES.ERROR);
					}
				})
			});
		},

		/**
		 * Refresh the import history table
		 */
		_refresh : function() {
			historyMessageBox.hideMessage();
			this.styledTable.clearBody();
			this._loadimporthistory();
		},
		/**
		 * get schedule info
		 */
		_getScheduleInfo:function(raw){
			var schedule={};
			var nextPlannedDate;
			if(raw.lastSuccessTime=="-"){//$NON-NLS-1$
				schedule.lastTimeMessage = messages.firstRecord;
				}
			else{
				schedule.lastTimeMessage = this._getLocalTime(raw.lastSuccessTime);
			}
			if(raw.scheduleMode==sadmConstants.SA_IMPORT_HISTORY_SCHEDULE_REPEAT_INTERVAL){
				schedule.detail = dojo.string.substitute(messages.repeatInterval,
	                    {intervalTime:this._getDuration(parseInt(raw.repeatTime)*1000)});
				if(raw.startTime =="-"){//have no startTime-->plannedTime=requestTime+delayTime
					nextPlannedDate =new Date(this._handlePlannedTime(raw.requestTime,raw.repeatTime));
				}else{ //repeat interval ,have the startTime-->plannedTime=startTime+delayTime
					nextPlannedDate =new Date(this._handlePlannedTime(raw.startTime,raw.repeatTime));
				}
				schedule.nextDate=dojo.date.locale.format(
						nextPlannedDate,{formatLength: "medium"});//$NON-NLS-1$
			}else if(raw.scheduleMode==sadmConstants.SA_IMPORT_HISTORY_SCHEDULE_REPEAT_WEEKLY){
				schedule.detail = dojo.string.substitute(messages.repeatWeeklyDetail,
	                     {gmtTime:raw.repeatTime,
					    week:this._getWeek(raw.repeatDays)});
				if(raw.startTime =="-"){//have no startTime-->plannedTime=requestTime+delayTime
					nextPlannedDate =new Date(this._handlePlannedTime(raw.requestTime,raw.nextDelay));
				}else{ //weekly repeat ,have the startTime: plannedTime=startTime+delayTime 
					nextPlannedDate=new Date (this._handlePlannedTime(raw.startTime,raw.nextDelay));
				}
				schedule.nextDate=dojo.date.locale.format(
					nextPlannedDate, {formatLength: "medium"});//$NON-NLS-1$
					
			}else{
				schedule.detail=messages.repeatNo;
				schedule.nextDate="";//$NON-NLS-1$
			}
			return schedule;
		},
		/**
		 * get diagram type name list
		 */
		_getDiagramNameList:function(raw){
			var namelist = [];
			if (raw.importType == sadmConstants.SA_IMPORT_HISTORY_IMPORT_ALL) {
				listAll = [messages.all];
			} else if(raw.diagramType!=null){
				listAll = this
						._parseDiagramTypes(raw.diagramType);
			}
			if(listAll[0]==sadmConstants.EMPTYSTRING)
		    	return sadmConstants.EMPTYSTRING;
			var len = listAll.length>9? 9: listAll.length;
			for(var i =0;i<len;i++){
				var li = dojo.create("li", {innerHTML:listAll[i]});//$NON-NLS-1$
				namelist.push(this._generateOutHtml(li));
			}
			if( listAll.length>len){
				var more = dojo.create("li", {innerHTML:messages.suspensionPoints});//$NON-NLS-1$
				namelist.push(this._generateOutHtml(more));
			}
			return namelist.join("");//$NON-NLS-1$
		},
		/**
		 * get diagram instances 
		 */
		_getDiagramInstances:function(raw){
			var diagramInstances=[];
			if (raw.importType == sadmConstants.SA_IMPORT_HISTORY_IMPORT_ALL) {
				listAll = [messages.all];
			}else if(raw.diagramInstance!=null){
				listAll =this._parseDiagramInstances(raw.diagramInstance);
			}
			
			if(listAll[0]==sadmConstants.EMPTYSTRING)
		    	return sadmConstants.EMPTYSTRING;
			var len = listAll.length>9? 9: listAll.length;
			for(var i =0;i<len;i++){
				var li = dojo.create("li", {innerHTML:listAll[i]});//$NON-NLS-1$
				diagramInstances.push(this._generateOutHtml(li));
			}
			if( listAll.length>len){
				var more = dojo.create("li", {innerHTML:messages.suspensionPoints});//$NON-NLS-1$
				diagramInstances.push(this._generateOutHtml(more));
			}
			return diagramInstances.join("");//$NON-NLS-1$
		},
		_parseDiagramInstances:function(diagramInstancesString){
			var diagramInstances =[];
			diagramInstances=diagramInstancesString.slice(1,diagramInstancesString.length-1).split(","); //$NON-NLS-1$
			return diagramInstances;
		},
		/**
		 * get diagram type name 
		*/
		_parseDiagramTypes : function(typeListString) {
			var array = [];
			var typeList = [];
			array = typeListString.slice(1,typeListString.length-1).split(",");//$NON-NLS-1$
			
			for ( var i = 0; i < array.length; i++) {
				typeList[i] = array[i].substr(array[i]
						.lastIndexOf("/"), array[i].length);//$NON-NLS-1$
			}
			var types = [];
			var temp = [];
			for ( var j = 0; j < typeList.length; j++) {
				temp[j] = typeList[j].replace(new RegExp(
						"/", "g"), "");//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
				types[j] = this._restoreSATypeName(temp[j]);
			}
			return types;
		},
		/**
		 * restore SA diagram type name
		 */
		_restoreSATypeName : function(sName) {
			var retval = sName;
			for( var ent in saEntities ) {
				retval = retval.replace( new RegExp(ent,"g") , saEntities[ent] ); //$NON-NLS-1$ 
			}
			return retval;
		},
		
		/**
		 * get encyclopedia name form SA context info
		 */
		_praseEncyName:function(saContext){
			temp = saContext.substr(0,saContext.lastIndexOf("/"));//$NON-NLS-1$ 
			return temp.substr(temp.lastIndexOf("/")+1,temp.length).replace(new RegExp("%20","g")," "); //$NON-NLS-1$ 
		},
			
						
		/**
		 * Get All the import history records from the Storage
		 */
		_loadimporthistory : function(result) {
			if(result)
				this.permissionChk = result;
			var query = {};
			var context = jazz.app.context.get();
			var uri = net.jazz.ajax._contextRoot + "/ImportHistory";//$NON-NLS-1$
			query.projectId = context.itemId;
			//if isCheckReboot is true, it needs reboot check in the server side, else it needn't 
			query.isCheckReboot=this.isFirstVisit;
			query.beginNumber=(this.currentPage-1) * this.pageSize;
			query.endNumber=query.beginNumber + this.pageSize;
			if(this.isFirstVisit==true)
			this.isFirstVisit=false;
			this.loadingImageArea.style.display = "block"; //$NON-NLS-1$ 
            jazz.client.xhrGet({
				url : uri + "?" + dojo.objectToQuery(query),//$NON-NLS-1$
				headers : {
					"Accept" : "text/xml",//$NON-NLS-1$ $NON-NLS-2$
					"Content-Type" : "text/html",//$NON-NLS-1$ $NON-NLS-2$
					"X-ibm-rmps-internal" : "true" //$NON-NLS-1$ $NON-NLS-2$
				},
				content : {
					data : null
				},

				load : dojo.hitch(this, this._handleJSON),
				error : dojo.hitch(this, function(result) {
					this.loadingImageArea.style.display = "none"; //$NON-NLS-1$ 
				})
			});
		},
		
		/**
		 * Check the permission of cancel and delete of user
		 */
		_checkPermission : function(){
			var query = {};
			var context = jazz.app.context.get();
			var uri = net.jazz.ajax._contextRoot + "/ImportHistory";//$NON-NLS-1$
			query.projectAreaContext = context.uri;
			jazz.client.xhrPost({
				url : uri + "?" + dojo.objectToQuery(query),//$NON-NLS-1$
				headers : {		
					"Accept" : "text/xml",//$NON-NLS-1$ $NON-NLS-2$
					"Content-Type" : "text/html",//$NON-NLS-1$ $NON-NLS-2$
					"X-ibm-rmps-internal" : "true" //$NON-NLS-1$ $NON-NLS-2$
				},
				content : {
					data : context,
				},

				load : dojo.hitch(this, function(result){this._loadimporthistory(result);}),
				error : dojo.hitch(this, function(){this._loadimporthistory();}),
			});
		},

		/**
		 * Handle the returned JSON data
		 */
		_handleJSON : function(jsonString) {
			this.loadingImageArea.style.display = "none"; //$NON-NLS-1$ 
			// clear the rawInfo
			this.rawInfo.length = 0;
			var data = eval("(" + jsonString + ")"); //$NON-NLS-1$ //$NON-NLS-2$ 
			this._updatePage(data.sum);
			var jsonObject=data.importHistory;
			this.rawInfo=this._parseJSON(jsonObject);
			this.rawInfo.reverse();
			this._showPage();

		},
		
		_parseJSON:function(jsonobject){
			var raws=[];
			for ( var key in jsonobject) {
				var raw = {
					key:key,
					progress:jsonobject[key].progress,
					requester:jsonobject[key].requester,
					requestTime:jsonobject[key].requestTime,
					startTime:jsonobject[key].startTime,
					duration:jsonobject[key].duration,
					stage:jsonobject[key].stage,	
					scheduleMode:jsonobject[key].scheduleMode,
					repeatDays:jsonobject[key].repeatDays,
					saContext:jsonobject[key].saContext,
					saWorkspace:jsonobject[key].saWorkspace,
					repeatTime:jsonobject[key].repeatTime,
					diagramType:jsonobject[key].diagramType,
					diagramInstance:jsonobject[key].diagramInstance,
					importType:jsonobject[key].importType,
					lastSuccessTime:jsonobject[key].lastSuccessTime,
					fullLog:jsonobject[key].fullLog,
					nextDelay:jsonobject[key].nextDelay,
					streamName:jsonobject[key].streamName,
					executionException:jsonobject[key].executionException,
					snapshotName:jsonobject[key].snapshotName,
					snapshotUrl:jsonobject[key].snapshotUrl,
					cancelInfo:jsonobject[key].cancelInfo,
				};
				raws.push(raw);
			}
			return raws;
		},

		/**
		 * Get the related Progress Icon according to the Progress String
		 */
		_getProgressIcon : function(progress) {
			var icon = null;
			switch (progress) {
			case sadmConstants.SA_IMPORT_PROGRESS_COMPLETED:
				icon = PROGRESS_COMPLETED;
				break;
			case sadmConstants.SA_IMPORT_PROGRESS_WAITING:
				icon = PROGRESS_WAITING;
				break;
			case sadmConstants.SA_IMPORT_PROGRESS_RUNNING:
				icon = PROGRESS_RUNNING;
				break;
			case sadmConstants.SA_IMPORT_PROGRESS_FAILED:
				icon = PROGRESS_BROKEN;
				break;
			case sadmConstants.SA_IMPORT_PROGRESS_CANCELED:
				icon = PROGRESS_CANCELED;
				break;
			case sadmConstants.SA_IMPORT_PROGRESS_CANCELING:
				icon = PROGRESS_RUNNING;
				break;
			default:

			}
			return icon;
		},

		_showPage : function() {
			if(!this._updateTimer){
				this._updateTimer = setInterval(dojo.hitch(this, this._autoUpdate), sadmConstants.AUTO_REFRESH_TIME);
			}
			this.styledTable.clearBody();
			dojo.forEach(this.rawInfo, function(raw) {
				this.styledTable.tbody.appendChild(this._createTr(raw));
			}, this);
		},
		
		/**
		 * update the page control infomation
		 */
		_updatePage:function(sum){
			if(this.sumImportHistory == sum)
				return;
			this.sumImportHistory = sum;
			this.sumPage = Math.ceil(sum/this.pageSize);
			var pageControl = new jazz.ui.PaginationControl({
				increment:this.pageSize,
				startPage:this.currentPage-1,
				total:this.sumImportHistory,
				onNextClick:dojo.hitch(this,this._nextPage),
				onPreviousClick:dojo.hitch(this,this._previousPage),
			});
			if(this.pageControlArea.childNodes.length > 0)
				this.pageControlArea.replaceChild(pageControl.domNode,this.pageControlArea.firstChild);
			else 
				this.pageControlArea.appendChild(pageControl.domNode);
		},
		
		_nextPage:function(){
			if(this.currentPage < this.sumPage)
				this.currentPage++;
			this._checkPermission();
		},

		_previousPage:function(){
			if(this.currentPage > 1)
				this.currentPage--;
			this._checkPermission();
		},
		
		/**
		 * Get the formated Local Time from the UTCString
		 */
		_getLocalTime : function(UTCString) {
			if (UTCString === '-') {//$NON-NLS-1$
				return UTCString;
			}else if(UTCString !=null){
			var UTCTime = dojo.date.stamp.fromISOString(UTCString);
			var localTime = new Date(UTCTime.valueOf()- (new Date().getTimezoneOffset()) * 60000);
			var formatTime = dojo.date.locale.format(localTime, {formatLength: "medium"});//$NON-NLS-1$//$NON-NLS-2$
			return formatTime;
			}
		},
		
		/**
		 * handle the repeat Days
		 */
		_getWeek:function(repeatDays){
			var weeks =[messages.sunday,messages.monday,messages.tuesday,messages.wednesday,messages.thursday,messages.friday,messages.saturday];
			var week=[];
			if(repeatDays[0]==1){
			 week[0] = weeks[0];
			}
			for(var i=1;i<repeatDays.length;i++){
				if(repeatDays[i]==1){
					 week=week.concat(weeks[i]);
				}
			}
			return week;
		},
		/**
		 * covert the UTCString time to Date
		 */
		_convertTime : function(UTCString) {
			if (UTCString =="-") {//$NON-NLS-1$
				return UTCString;
			}
			var UTCTime = dojo.date.stamp
					.fromISOString(UTCString);
			var DateTime = new Date(UTCTime.valueOf()
					- (new Date().getTimezoneOffset()) * 60000);

			return DateTime;
		},
		/**
		 * Get the formated duration from duration ms, here the
		 * duration accurate to second
		 */
		
		_getDuration : function(totalMs) { 
			var duration = "";//$NON-NLS-1$
			var spaceString = " ";//$NON-NLS-1$
			if (totalMs < 1000) {//$NON-NLS-1$
				return parseInt(totalMs) + spaceString+ messages.millisecond;
			}
			var totalSeconds = parseInt(totalMs / 1000);
			var seconds = totalSeconds % 60;
			var totalMinutes = parseInt(totalSeconds / 60);
			var minutes = totalMinutes % 60;
			var totalhours = parseInt(totalMinutes / 60);
			var hours = totalhours % 24;
			var days = parseInt(totalhours / 24);
			if (days !== 0) {
				if(days === 1){
				   duration += days + spaceString + messages.day + spaceString;
				}else{
				   duration += days + spaceString + messages.days + spaceString;	
				}
			}
			if (hours !== 0) {
				if(hours === 1){
				  duration += hours + spaceString + messages.hour + spaceString;
				}else{
				  duration += hours + spaceString + messages.hours + spaceString;
				}

			}
			if (minutes !== 0) {
				if(minutes === 1){
				  duration += minutes + spaceString + messages.minute + spaceString;
				}else{
				  duration += minutes + spaceString + messages.minutes + spaceString;	
				}
			}
			if (seconds !== 0) {
				if(seconds === 1){
				  duration += seconds + spaceString + messages.second;
				}else{
				  duration += seconds + spaceString + messages.seconds;	
				}
			}
			return duration;
		},
		/*
		 * get SADM now server URI 
		 */
		_getSADMServerURI:function(){
			var URI = window.location.href;
			var splitArr = new Array();
			splitArr = URI.split("/dm/");//$NON-NLS-1$
			if(splitArr[0] != URI)
				return splitArr[0] + "/";//$NON-NLS-1$ 
			else 
				return null;
		},
		
		/*
		 *  to generate outHTML of dom node
		 */
		_generateOutHtml:function(node){
			return sadmConstants.LEFT_TAG_OF_DOM_NODE + node.nodeName + sadmConstants.RIGHT_TAG_OF_DOM_NODE 
						+ node.innerHTML+
					sadmConstants.LEFT_TAG_OF_DOM_NODE + sadmConstants.SLASH + node.nodeName + sadmConstants.RIGHT_TAG_OF_DOM_NODE 	
		}

	});
})();