/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.rational.psm.importer.temp                                     |
 *|                                                                        |
 *| Copyright IBM Corporation 2011-2013.  All Rights Reserved.             |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corporation.          |
 *+------------------------------------------------------------------------+
 */


(function(){
	/**
	 * This widget provides the Diagram Types' Grid
	 * 
	 * @author Kai Dong
	 */
	dojo.provide("psm.importer.ui.internal.pages.SAImportSelection");
		
	dojo.require("psm.importer.ui.internal.pages.SAImportBySearching");
	dojo.require("psm.importer.ui.internal.pages.SelectDiagramsTreeWidget");
	dojo.require("psm.common.constants.sadmConstants");
	dojo.requireLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");


	// For which radio button the user selected
	var RADIO_BUTTON_IMPORT_ALL = 1;
	var RADIO_BUTTON_IMPORT_SELECTED = 2;
	var RADIO_BUTTON_IMPORT_SEARCHED = 3;
    var messages = dojo.i18n.getLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");
	var sadmConstants = psm.common.constants.sadmConstants;

    
    dojo.declare("psm.importer.ui.internal.pages.SAImportSelection", [dijit._Widget, dijit._Templated], {
    	messages : messages,
    	serverUri : null,   
    	importActionButton: null,
    	templateString :  
    		dojo.cache("psm.importer", "ui/internal/pages/templates/SAImportSelection.html"),
    		
    	constructor: function(){  
        	this.serverUri = arguments[0].serverUri;
        	this.importActionButton = arguments[0].button;
        }, 
        
    	postCreate : function(){    		
    		this.inherited(arguments);  
    		this.connect(this.radioButtonForSelectDiagram1,"onclick",dojo.hitch(this,this._radioSelect));//$NON-NLS-1$
    		this.connect(this.radioButtonForSelectDiagram2,"onclick",dojo.hitch(this,this._radioSelect));//$NON-NLS-1$
    		this.connect(this.radioButtonForSelectDiagram3,"onclick",dojo.hitch(this,this._radioSelect));//$NON-NLS-1$
    		this.importActionButton.disabled = sadmConstants.EMPTYSTRING;	
    		this.tree = null;
    		this.search = null;
    	},

    	/**
    	 * handle button event after radio button is clicked
    	 */
    	_radioSelect : function(e){
    		if(dojo.byId("selectForm").elements[0].checked){//$NON-NLS-1$
				this.searchInstanceAttachPoint.style.display="none";
    			this.diagramTreeAttachPoint.style.display="none";
    			this.importActionButton.disabled = sadmConstants.EMPTYSTRING;     			
    		} else if(dojo.byId("selectForm").elements[1].checked){//$NON-NLS-1$
				if(this.tree == null){
    				this.tree = new psm.importer.ui.internal.pages.SelectDiagramsTreeWidget({
    					serverUri:this.serverUri, 
    					button:this.importActionButton});
        			this.importActionButton.disabled = "disabled";     			
				} else {
    				this.tree.updateImportButton();
				}
				this._replaceChild(this.diagramTreeAttachPoint, this.tree.domNode);
				this.searchInstanceAttachPoint.style.display="none";
    			this.diagramTreeAttachPoint.style.display="block";
    		} else if(dojo.byId("selectForm").elements[2].checked){//$NON-NLS-1$
    			if(this.search == null){
    				this.search = new psm.importer.ui.internal.pages.SAImportBySearching({
    					serverUri:this.serverUri,
    					button:this.importActionButton});
        			this.importActionButton.disabled = "disabled";     			
 				} else {
    				this.search.updateImportButton();
				}
   				this._replaceChild(this.searchInstanceAttachPoint, this.search.domNode);
   				this.searchInstanceAttachPoint.style.display="block";
    			this.diagramTreeAttachPoint.style.display="none";
    						
    		}

    		 
    	},
    	
    	/**
    	 * get which radio button is selected
    	 */
    	getSelectedRadio:function(){
    		if(dojo.byId("selectForm").elements[0].checked){//$NON-NLS-1$
    			return RADIO_BUTTON_IMPORT_ALL;
    		}
    		if(dojo.byId("selectForm").elements[1].checked){//$NON-NLS-1$
    			return RADIO_BUTTON_IMPORT_SELECTED;
    		}
    		if(dojo.byId("selectForm").elements[2].checked){//$NON-NLS-1$
    			return RADIO_BUTTON_IMPORT_SEARCHED;		
    		}
    	},
    	
    	/**
		 * replace child ,first remove all children,then append child.
		 */
		_replaceChild : function( parent , child) {
			while( parent.firstChild != null )
				parent.removeChild( parent.firstChild );			
			parent.appendChild(child);
		},	

		/**
		 * get the selected types and instances xml 
		 */
		getSelectedTypesAndInstancesXML:function(){
			var typesList = [];
			var instancesList = [];
			if(this.getSelectedRadio() == RADIO_BUTTON_IMPORT_SELECTED){				
				var storeOfModel = this.tree.storeOfModel;
				var modelOfTree = this.tree.modelOfTree;
				var allItems = storeOfModel._arrayOfAllItems;
				dojo.forEach(allItems,function(item){
					if(item!=null){
						if( storeOfModel.getValue(item, "type") == "parent"){//$NON-NLS-1$
							if(storeOfModel.getValue(item, "mixed") == true){	//$NON-NLS-1$			
								modelOfTree.getChildren(item,dojo.hitch(this,function(children){
									dojo.forEach( children, function( child ){
										if(storeOfModel.getValue(child,"checkbox") == true){//$NON-NLS-1$
											instancesList.push("<diagramInstance title=\""+child.originName+"\" uri=\""+child.uri+"\" />");//$NON-NLS-1$;
										}
									},this);
								}),null);

							}else{
								if(storeOfModel.getValue(item,"checkbox") == true){//$NON-NLS-1$
									typesList.push("<diagramType uri=\""+item.uri+"\" />");//$NON-NLS-1$
								}
							}
						}
					}

				},this);
			} else if (this.getSelectedRadio() == RADIO_BUTTON_IMPORT_SEARCHED) {
				var selectedItems = this.search.getSelectedDiagrams();
				dojo.forEach(selectedItems,function(item){
					if(item!=null){
						instancesList.push("<diagramInstance title=\""+item.name+"\" uri=\""+item.uri+"\" />");//$NON-NLS-1$;
					}
				},this);
				
			}
			
			var typesAndInstancesXml = "<diagramTypes>" + typesList.join("\n") + "</diagramTypes>" +//$NON-NLS-1$ //$NON-NLS-2$
										"<diagramInstances>" + instancesList.join("\n") + "</diagramInstances>";//$NON-NLS-1$ //$NON-NLS-2$
			// By default, import option is upgrade;
			var importOption = sadmConstants.SA_IMPORT_OPTION_UPDATE;
			// By default, import all diagram instance without user selection
			var importType = sadmConstants.SA_IMPORT_TYPE_ALL;
			if(this.getSelectedRadio() != RADIO_BUTTON_IMPORT_ALL) {
				importType = sadmConstants.SA_IMPORT_TYPE_SELECTED;
			}
			var xmlResult = "<resources importType=\"" + importType	+ "\" importOption=\"" + importOption + "\">  " + //$NON-NLS-1$
								typesAndInstancesXml +
							"</resources>" ;//$NON-NLS-1$
			return xmlResult;
			
		},
    });
   
})();