/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.rational.psm.importer.temp                                     |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corporation.          |
 *+------------------------------------------------------------------------+
 */

(function() {
	/**
	 * This widget provides schedule section for SA import request
	 * 
	 * @author JunPing Cai & Kai Dong
	 */
	dojo.provide("psm.importer.ui.internal.pages.ScheduleWidget");
	dojo.require("dojo.cache");
	dojo.require("dojo.i18n");
	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit.form.ValidationTextBox");
	dojo.require("psm.common.constants.sadmConstants");
	dojo.require("dijit.form.CheckBox");
	dojo.requireLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");
	var messages = dojo.i18n.getLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");
	var sadmConstants = psm.common.constants.sadmConstants;
	dojo.declare("psm.importer.ui.internal.pages.ScheduleWidget", [ dijit._Widget, dijit._Templated ], {
		messages : messages,
		templateString : dojo.cache("psm.importer", "ui/internal/pages/templates/ScheduleWidget.html"),
		constructor : function() {
			this.weekDayCheckBox = new Array();			
			this.isRepeatCheckBox = null;
		},

		postCreate : function() {
			this._init();
			this._disableRepeatSelector();
			dojo.connect(this.isRepeat, "onclick", dojo.hitch(this, this._operateRepeatModule)); //$NON-NLS-1$	
			dojo.connect(this._repeatInHoursRadio, "onclick", dojo.hitch(this, this._enableRepeatInHours));//$NON-NLS-1$
			dojo.connect(this._repeatSpecificRadio, "onclick", dojo.hitch(this, this._enableRepeatSpecificSelector)); //$NON-NLS-1$
		},

		/**
		 * user can not select any RadioButton and can not input or select
		 * specific time.
		 */
		_disableRepeatSelector : function() {
			this._repeatInHoursRadio.disabled = true;
			this.inputCustomHours.setDisabled(true);
			this._repeatSpecificRadio.disabled = true;
			this._repeatSpecificHoursSelect.disabled = true;
			this._repeatSpecificMinutesSelect.disabled = true;
			this._disableDaysCheckBox(true);
		},

		/**
		 * initial the Time selector
		 */
		_init : function() {
			this.isRepeatCheckBox = new dijit.form.CheckBox();
			this.isRepeat.appendChild(this.isRepeatCheckBox.domNode);
			this.inputCustomHours = new dijit.form.ValidationTextBox({
				regExp : "^([1-9][0-9]*(\\.[0-9]+)?|0\\.(?!0+$)[0-9]+)$", //$NON-NLS-1$
				value : "1.00",//$NON-NLS-1$
				Class : "inputIntervalClass"//$NON-NLS-1$
			});
			this._repeatInHoursInput.appendChild(this.inputCustomHours.domNode);
			// init Hours selector
			for ( var i = 0; i < 24; ++i) {
				var value = i;
				if (i < 10) {
					value = "0" + i;//$NON-NLS-1$
				}
				this._createOption(this._repeatSpecificHoursSelect, value, value, i == 0);
			}
			// init minute selector
			for ( var i = 0; i < 60; ++i) {
				var value = i;
				if (i < 10) {
					value = "0" + i;//$NON-NLS-1$
				}
				this._createOption(this._repeatSpecificMinutesSelect, value, value, i == 0);
			}

			this._repeatSpecificTimeZoneOffset.appendChild(dojo.doc.createTextNode(messages.repeatGMT));
			// add days CheckBox
			var weekDays = [ messages.sunday, messages.monday, messages.tuesday, messages.wednesday, messages.thursday,
					messages.friday, messages.saturday ];
			for(var i = 0; i < 7; i++){
				this.weekDayCheckBox[i] = new dijit.form.CheckBox({
					name : "repeatWeekDay",//$NON-NLS-1$
					id : "weekDays_"+i,//$NON-NLS-1$
					value:Math.pow(2,i),
					disabled : "disabled"//$NON-NLS-1$
				});
				
				this._appendChildToRepeatSpecificDays(this.weekDayCheckBox[i],weekDays[i]);
			}			
		},

		/**
		 * if the isRepeat CheckBox is checked then enable the RadioButtons else
		 * disable to check the RadioButtons
		 * 
		 */
		_operateRepeatModule : function() {
			if (this.isRepeatCheckBox.checked) {
				this._enableRepeatRadioButtons();
				this.inputCustomHours.setDisabled(false);
				this._repeatInHoursRadio.checked = true;
			} else {
				this._disableRepeatSelector();
			}
		},

		/**
		 * user can input in the TextBox of the first RadioButton and user can
		 * not select specific time in the second RadioButton
		 */
		_enableRepeatInHours : function() {
			this.inputCustomHours.setDisabled(false);
			this._repeatSpecificHoursSelect.disabled = true;
			this._repeatSpecificMinutesSelect.disabled = true;
			this._disableDaysCheckBox(true);
		},

		/**
		 * user can only select specific time in the second RadioButton and user
		 * can not input in the TextBox of the first RadioButton
		 */
		_enableRepeatSpecificSelector : function() {
			this.inputCustomHours.setDisabled(true);
			this._repeatSpecificHoursSelect.disabled = false;
			this._repeatSpecificMinutesSelect.disabled = false;
			this._disableDaysCheckBox(false);

		},

		/**
		 * if user checked the the isRepeat CheckBox then call this method to
		 * enable RadioButtons
		 */
		_enableRepeatRadioButtons : function() {
			this._repeatInHoursRadio.disabled = false;
			this._repeatSpecificRadio.disabled = false;
		},

		/**
		 * create Options in selector
		 */
		_createOption : function(selectNode, label, value, selected) {
			var option = dojo.create("option");//$NON-NLS-1$
			option.value = value;
			option.appendChild(dojo.doc.createTextNode(label));
			if (selected) {
				option.selected = true;
			}
			selectNode.appendChild(option);
		},
		/**
		 * Disable the Week Days CheckBox
		 * @param flag
		 */
		_disableDaysCheckBox : function(flag) {
			for(var i = 0;i < 7; i++){
				this.weekDayCheckBox[i].setDisabled(flag);
			}			
		},
		
		/**
		 * append the name of each CheckBox
		 * @param day
		 * @param name
		 */
		_appendChildToRepeatSpecificDays : function(day,name) {
			this._repeatSpecificDays.appendChild(day.domNode);
			this._repeatSpecificDays.appendChild(dojo.create("span", {//$NON-NLS-1$
				innerHTML : name,
				Class : "repeatDaysBlankStyle"//$NON-NLS-1$
			}));
		},
		/**
		 * get the repeat week days it will return a int : the sum value of the selected checkBox
		 * @returns
		 */
		getRepeatDays : function() {
			selectedDays = 0;
			for( var i = 0; i < 7; i ++) {			
				selectedDays = selectedDays + (this.weekDayCheckBox[i].checked? this.weekDayCheckBox[i].value:0);
			}
			return selectedDays;
		},
		/**
		 * get the repeat Mode if user check the first RadioButton it will return 1;
		 * if user check the second RadioButton it will return 2;
		 * else  it will return 0;
		 * @returns {String}
		 */
		getRepeatMode : function(){
			if(this.isRepeatCheckBox.checked){
				if(this._repeatInHoursRadio.checked){
					return sadmConstants.SA_IMPORT_SCHEDULE_REPREAT_INTERVAL;//$NON-NLS-1$
				}else{
					return sadmConstants.SA_IMPORT_SCHEDULE_REPEAT_WEEKLY;//$NON-NLS-1$
				}
			}else{
				// no need to repeat import
				return sadmConstants.SA_IMPORT_SCHEDULE_REPEAT_OFF;//$NON-NLS-1$
			}
		},
		/**
		 * return  the repeat Time if user select the second RadioButton
		 * @returns {String}
		 */
		getRepeatTime : function(){
			var hour = this._repeatSpecificHoursSelect.value;
			var minute = this._repeatSpecificMinutesSelect.value;
			return hour+":"+minute;//$NON-NLS-1$
		},
		/**
		 * return the input value which user input.
		 * the import task will repeat every N seconds
		 * @returns {Number}
		 */
		getRepeatInputTime : function(){
			// return second of delayTime;
			return parseFloat(this.inputCustomHours.value)*60*60;
		},
		_isRepeatDayNotValid : function(repeatDays){
			var flag = true;
			for(var i = 0;i<7;i++){
				if(repeatDays[i] == "1"){
					flag = false;
					break;
				}					
			}
			return flag;
		},
		/**
		 * judge is the input valid or not
		 * @returns {Boolean}
		 */
		isInputValid : function(){
			var flag  = true;
			var scheduleMode = this.getRepeatMode();
			if(scheduleMode == sadmConstants.SA_IMPORT_SCHEDULE_REPREAT_INTERVAL){
				if(!this.inputCustomHours.isValid()||this.inputCustomHours.value ==""){//$NON-NLS-1$
					flag = false;
				}
			}else if(scheduleMode == sadmConstants.SA_IMPORT_SCHEDULE_REPEAT_WEEKLY){
					if(this.getRepeatDays() == 0 ){
						flag = false;
					}
			}			
			return flag;
		}

	});
})();
