/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.rational.psm.importer.temp                                     |
 *|                                                                        |
 *| Copyright IBM Corporation 2011-2013.  All Rights Reserved.             |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corporation.          |
 *+------------------------------------------------------------------------+
 */


(function(){
	/**
	 * This widget provides the Diagram Types' Tree 
	 * 
	 * @author Kai Dong
	 */
	dojo.provide("psm.importer.ui.internal.pages.SelectDiagramsTreeWidget");
	dojo.require("psm.importer.ui.internal.widget.checkboxtree.model.CheckBoxStoreModel");
	dojo.require("psm.importer.ui.internal.widget.checkboxtree.CheckBoxTree");
	dojo.require("psm.importer.ui.internal.constants.iconConstants");
	dojo.requireLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");
	dojo.require("psm.importer.ui.internal.pages.SACommonUtils");
	
	var messages = dojo.i18n.getLocalization("psm.importer.ui.internal.pages", "ApplicationMessages");	
	var iconConstants = psm.importer.ui.internal.constants.iconConstants;
	var saCommonUtils = psm.importer.ui.internal.pages.SACommonUtils;
	var LOADING_IMG = "<img class='importLoadingImage' title='"+messages.processNow+"' alt='"+messages.processNow //$NON-NLS-1$ //$NON-NLS-2$
	+ "' src="+iconConstants.LOADING_IMG+">"; //$NON-NLS-1$
	
    dojo.declare("psm.importer.ui.internal.pages.SelectDiagramsTreeWidget", [dijit._Widget, dijit._Templated], {
    	serverUri : null,   
    	importActionButton : null,   
    	messages : messages,
    	listAll : [],
    	storeOfModel : null,
    	modelOfTree : null,
    	diagramTypesTree : null,
    	templateString :  
    		dojo.cache("psm.importer", "ui/internal/pages/templates/SelectDiagramsTreeWidget.html"),
    		
    	constructor: function(){  
        	this.importActionButton = arguments[0].button;
        	this.serverUri = arguments[0].serverUri;
        },
        
    	postCreate : function(){    		
    		this.inherited(arguments);
    		this.importActionButton.disabled = "";//$NON-NLS-1$ 
    		this._constructTree();
    	},
    	
		/**
		 * to generate the structure of diagram type tree
		 */
		_constructTree:function(){
			if(this.modelOfTree==null){
				var server_uri = this.serverUri+"/Diagrams?hasInstances=true";//$NON-NLS-1$
	    		jazz.client.xhrGet({
		        	url: server_uri,	        	
		        	handleAs: "xml", //$NON-NLS-1$
		        	sync:true,
		        	load: dojo.hitch(this,function(result){
		        		if(!result){
							return;
						}
						var listOfType = saCommonUtils.parseDiagramTypesOrInstances(result);
						this._parseTypeListIntoTreeItems(listOfType);
						this.callback(this.listAll);
		        	}),
		        	error: dojo.hitch(this, function(error) {
					})
		        });
	    		
				var dataOfStore = {"identifier":"uri","label":"name","items":this.listAll};//$NON-NLS-1$
				this.storeOfModel = new dojo.data.ItemFileWriteStore( {
	                data : dataOfStore,
	            });
				this.modelOfTree = new psm.importer.ui.internal.widget.checkboxtree.model.CheckBoxStoreModel( {
	                store: this.storeOfModel,
	                query: {type: 'parent'},//$NON-NLS-1$
	                });
				
			}
			this.diagramTypesTree = new psm.importer.ui.internal.widget.checkboxtree.CheckBoxTree( {
                model: this.modelOfTree,                                    
                showRoot:false,
                });	
			this.treeAttachpoint.appendChild(this.diagramTypesTree.domNode);
			dojo.connect(this.diagramTypesTree, "_onExpandoClick",dojo.hitch( this , "_addChildrenToParent" ));//$NON-NLS-1$
			dojo.connect(this.diagramTypesTree,"onNodeChecked",dojo.hitch(this,"updateImportButton"));//$NON-NLS-1$
			dojo.connect(this.diagramTypesTree,"onNodeUnchecked",dojo.hitch(this,"updateImportButton"));//$NON-NLS-1$
		},		

		/**
		 *  update import button ,to disable import button or enable import button
		 */
		updateImportButton:function(){
			if(this._isHaveCheckedItem()){
				this.importActionButton.disabled="";//$NON-NLS-1$
			}else{
				this.importActionButton.disabled="disabled";//$NON-NLS-1$
			}
		},
		
		/**
		 * judge is any diagram type or diagram instance is checked
		 * 
		 */
		_isHaveCheckedItem:function(){
			var allItems = this.storeOfModel._arrayOfAllItems;
			return(dojo.some(allItems,function(item){
				if(item!=null){
					return item.checkbox == "true";//$NON-NLS-1$
				}
			}));
		},
		
		/**
		 * add children to parentItem
		 */
		_addChildrenToParent:function(message){
			var node = message.node;			
			var parentItem = node.item;
			if(this.storeOfModel.getValue(parentItem,"already_expanded") == false){//$NON-NLS-1$	
				loadingMessage = dojo.create("div",{style:{display:"inline"},innerHTML:LOADING_IMG},node.labelNode,"after"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				var jsonData = jazz.client.xhrGet({
		                url: parentItem.uri,             
		                handleAs: "xml", //$NON-NLS-1$
		                load: dojo.hitch(this,function(result){
		                	this._addDiagramInstancesIntoTree(result,parentItem);		                		
		                	loadingMessage.setAttribute("style","display:none");//$NON-NLS-1$ //$NON-NLS-2$
		                }),
		        });
				
			}
			
		},
		
		/**
		 * get diagram type file,and parse all diagram instances to store item,then add it as a child of parentItem
		 */
		_addDiagramInstancesIntoTree:function(result,parentItem){
			var checkedOrNot = this.storeOfModel.getValue(parentItem,"checkbox");//$NON-NLS-1$
			var instancesList = saCommonUtils.parseDiagramTypesOrInstances(result);
    		dojo.forEach(instancesList,dojo.hitch(this,function(item){
    			var tempItem = {
						name:item.name,
						originName:saCommonUtils.encodeSAName(item.name),
						uri:item.uri,
						checkbox:checkedOrNot,
						mixed:false,
						type:"child",//$NON-NLS-1$
				}
    			this.modelOfTree.newItem(tempItem,parentItem);
			}));
    		this.storeOfModel.setValue(parentItem,"already_expanded",true);//$NON-NLS-1$
		},
		
		/**
		 * parse diagram type list into tree items
		 */		
		_parseTypeListIntoTreeItems: function(list){
			dojo.forEach(list,dojo.hitch(this,function(item){
				var temp = {
						name:item.name+"("+item.instanceCount+")",//$NON-NLS-1$
						uri:item.uri,
						already_expanded:false,
						checkbox:false,
						mixed:false,
						type:"parent",//$NON-NLS-1$

				}
				this.listAll.push(temp);
			}));
		},
    });
   
})();