/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

/**
 * @author Nanqun CHEN
 */

(function() {
	
	dojo.provide("psm.importer.ui.internal.pages.hoverDialog");
	dojo.require("jazz.ui.HoverPopup");
	
	dojo.declare("psm.importer.ui.internal.pages.hoverDialog",jazz.ui.HoverPopup,{
		/**
		 * @args json type
		 * e.g.
		 * args={
		 *		around:domNode,
		 *		icon:iconString,
		 *		title:title,
		 *		data:{
		 *		    subtitle1:{key1:value1,key2:value2},
		 * 		    subtitle2:{one_line_value:null},
		 *		    key:value
		 *		}
		 *	}
		 */
		constructor: function(args) {
			this.around = args.around;
			this.icon = args.icon;
			this.title = args.title;
			this.data = args.data;
			this.className = args.className;
			this._initialized();
		},
		createContents:function(popup){
			return this.content;
		},
		_initialized:function(){
			this.content = dojo.create("div",{"class":this.className});//$NON-NLS-1$ //$NON-NLS-2$
			this.containter = dojo.create("div",{"class":"content-container"});//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			var titleDiv = this._createTitle();
			this.appendContent(titleDiv);
			this.appendContent(this.containter);
			this._createTable();
		},
		appendContent:function(domNode){
			this.content.appendChild(domNode);
		},
		appendContainer:function(domNode){
			this.containter.appendChild(domNode);
		},
		/**
		 * add data to the table using this.addLine(json)
		 * @line json type:
		 * e.g.
		 * line={
		 *		subtitle1:{key1:value1,key2:value2},
		 * 		subtitle2:{one_line_value:null},
		 *		key:value
		 * }
		 */
		addLine:function(line){
			if(!this.table){
				this.table = dojo.create("table");//$NON-NLS-1$ 
				this.appendContainer(this.table);
			}
			if(line instanceof Object){
				var tr, td1, td2;
				for(var key in line){
					if(line[key] instanceof Object){
						var th = dojo.create("th",{innerHTML:key,"colspan":"2"});//$NON-NLS-1$
						this.table.appendChild(th);
						var subline = line[key];
						for(var subkey in subline){
							tr = dojo.create("tr");//$NON-NLS-1$
							if(subline[subkey]){
								td1 = dojo.create("td",{innerHTML:subkey, "class":"tdlabel"});//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
								td2 = dojo.create("td",{innerHTML:subline[subkey]});//$NON-NLS-1$
								tr.appendChild(td1);
								tr.appendChild(td2);
								this.table.appendChild(tr);
							}else{
								var tr = dojo.create("tr");//$NON-NLS-1$
								var td = dojo.create("td",{innerHTML:subkey,"colspan":"2"});//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
								tr.appendChild(td);
								this.table.appendChild(tr);
							}
						}
					}else{	
						tr = dojo.create("tr");//$NON-NLS-1$
						td1 = dojo.create("td",{innerHTML:key, "class":"tdlabel"});//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						td2 = dojo.create("td",{innerHTML:line[key]});//$NON-NLS-1$
						tr.appendChild(td1);
						tr.appendChild(td2);
						this.table.appendChild(tr);
					}
				}
			}else{	
				var tr = dojo.create("tr");//$NON-NLS-1$
				var td = dojo.create("td",{innerHTML:line,"colspan":"2"});//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				tr.appendChild(td);
				this.table.appendChild(tr);
			}
		},
		_createTitle:function(){
			var titleDiv = dojo.create("div",{//$NON-NLS-1$
				innerHTML :"<div>"+ this.icon//$NON-NLS-1$
					+ "<span class='title-log'>"//$NON-NLS-1$
					+ this.title
					+ "<div><div class='title-divider'></div></div>",//$NON-NLS-1$
				"class":"title-container"//$NON-NLS-1$ //$NON-NLS-2$
			});
			return titleDiv;
		},
		/**
		 * initialize the table and data when construct
		 */
		_createTable:function(){
			if(!this.data){
				return;
			}
			this.table = dojo.create("table");//$NON-NLS-1$
			
			var th, tr, td1, td2, dataArr;
			for(var key in this.data){
				th = dojo.create("th",{innerHTML:key});//$NON-NLS-1$
				this.table.appendChild(th);
				dataDetail = this.data[key];
				for(var key in dataDetail){
					tr = dojo.create("tr");//$NON-NLS-1$
					if(dataDetail[key]){
						td1 = dojo.create("td",{innerHTML:key, "class":"tdlabel"});//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						td2 = dojo.create("td",{innerHTML:dataDetail[key]});//$NON-NLS-1$
						tr.appendChild(td1);
						tr.appendChild(td2);
					}else{
						td1 = dojo.create("td",{innerHTML:key, "colspan":"2"});//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						tr.appendChild(td1);
					}
					this.table.appendChild(tr);
				}
			}
			this.appendContainer(this.table);
		}
	});
})();					