/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.rational.psm.sadiagram.web                                     |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corporation.          |
 *+------------------------------------------------------------------------+
 */

(function() {
	/**
	 * This widget provides a custom dijitTree whith checkbox
	 * 
	 * @author Kai Dong
	 */
	dojo.provide("psm.importer.ui.internal.widget.checkboxtree.CheckBoxTree");
	dojo.require("dijit.Tree");	
	dojo.require("dijit.form.CheckBox");
	dojo.require("psm.importer.ui.internal.widget.checkboxtree.CheckBox")

	dojo.declare( "psm.importer.ui.internal.widget.checkboxtree.CheckBoxTree", dijit.Tree,
	{

		openOnClick: true,
		hasChecked:false,

		postCreate:function(){
			var store = this.model.store;
			if(!store.getFeatures()['dojo.data.api.Write']){
				throw new Error("psm.importer.ui.internal.widget.checkboxtree.CheckboxTree: store must support dojo.data.Write");
			}	
			this.inherited(arguments);
		},	
		
		/**
		 * handle checkbox click event
		 **/
		_onClick: function( nodeWidget, e) {
			var domElement = e.target;
			if(domElement.nodeName != 'INPUT') {//$NON-NLS-1$
				return this.inherited( arguments );
			}
			this.model.updateCheckbox( nodeWidget.item, nodeWidget._checkbox.get("checked") );//$NON-NLS-1$
		    this.onClick( nodeWidget.item, nodeWidget, e );			
			if(nodeWidget._checkbox.get("checked")) {//$NON-NLS-1$
				this.onNodeChecked( nodeWidget.item, nodeWidget);
			} else {
				this.onNodeUnchecked( nodeWidget.item, nodeWidget);
			}
			this.focusNode(nodeWidget);
		},
		
		/**
		*	handle event when user click to expand node
		*	
		**/
		_onExpandoClick:function(message){
			this.inherited(arguments);
		},


		/**
		* it will be called when the checkbox is  checked and do something
		**/
		onNodeChecked: function( storeItem, nodeWidget) {
		},
		
		/**
		* it will be called when the checkbox is unchecked and do something
		**/
		onNodeUnchecked: function( storeItem, nodeWidget) {
		},
		
		/**
		*
		* override parent's method
		**/
		_createTreeNode:function( args ){
			// summary:
			//        creates a TreeNode
			// description:
			//                Developers can override this method to define their own TreeNode class;
			//                However it will probably be removed in a future release in favor of a way
			//                of just specifying a widget for the label, rather than one that contains
			//                the children too.		
			return new psm.importer.ui.internal.widget.checkboxtree._CheckBoxTreeNode(args);
		},
		
		/**
		* this is the call back if the model is modified.
		**/
		_onItemChange:function( storeItem ){
			this.inherited(arguments);		
			var model = this.model;
			var state = model.getCheckboxState( storeItem );
			var identity = model.getIdentity(storeItem);
			var nodes = this._itemNodesMap[identity];
			if( nodes ){
				dojo.forEach( nodes, function(node) {
					if( node._checkbox != null ) {
						node._setCheckedState( state );
					}
				}, this );
			}
		},
		

		
	});
	
	
	dojo.declare("psm.importer.ui.internal.widget.checkboxtree._CheckBoxTreeNode", dijit._TreeNode,
	{
		_checkbox:null,

		/**
		*   create a checkbox in expandNode
		**/
		_createCheckBox:function(){
			var  state = this.tree.model.getCheckboxState( this.item );
			if( state.checked !== undefined ) {		
				this._checkbox = new psm.importer.ui.internal.widget.checkboxtree.CheckBox().placeAt(this.expandoNode,'after');
				this._setCheckedState( state );
			}
			
		},
		
		/**
		 * set the label of tree node,hoverlink and other things to show
		 */
		_setLabelAttr:function(val){
			//add hover link on child(diagram instance)
			if(this.item.type == "child"){//$NON-NLS-1$
				var link = dojo.create("a");//$NON-NLS-1$
				link.href = this.item.uri;
				link.innerHTML = this.item.name;
				var  resLink = new jazz.ui.ResourceLink( { retainLinkText: true } , link );
				this.labelNode.innerHTML = "";//delete diagram name and add the hover link
				this.labelNode.appendChild(resLink.domNode);
			}else{
				this.labelNode[this.labelType == "html" ? "innerHTML" : "innerText" in this.labelNode ?//$NON-NLS-1$
                        "innerText" : "textContent"] = val;//$NON-NLS-1$
                this._set("label", val);//$NON-NLS-1$
			}
		},
		postCreate:function(){
			this._createCheckBox();
			if(this.item.type == "parent"){//$NON-NLS-1$
				this.isExpandable = true;
				this._setExpando(false);
			}
			this.inherited(arguments);
		},
		
		/**
		 * set the checkbox's state
		 */
		_setCheckedState: function(  state ) {
			this._checkbox._setCheckedState(state.checked,state.mixed);
		},

	});
	

	
})();