/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.rational.psm.sadiagram.web                                     |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corporation.          |
 *+------------------------------------------------------------------------+
 */
(function() {
	/**
	 * This is the store model of three status tree
	 * 
	 * @author Kai Dong
	 */
	dojo.provide("psm.importer.ui.internal.widget.checkboxtree.model.CheckBoxStoreModel");
	dojo.require("dijit.tree.ForestStoreModel");
	dojo.declare("psm.importer.ui.internal.widget.checkboxtree.model.CheckBoxStoreModel", dijit.tree.ForestStoreModel,
	{
		/**
		 * the identifier for checkbox
		 */
		checkboxIdent : "checkbox",//$NON-NLS-1$
		/**
		 * the mixed status of checkbox
		 */
		checkboxMState : "mixed",//$NON-NLS-1$
		/**
		 * default checkbox status
		 */
		checkboxState : false,

		/**
		 * The identity specified for the store. The identity equates to
		 * the 'identifier'
		 */
		_identity : null,

		/**
		 * update the checkbox state in store, it will be called by the
		 * tree whenever a checkbox is clicked. this is the way the tree
		 * will let the model know things have changed on the tree side.
		 * and this will update children's status and parent's status
		 */
		updateCheckbox : function( storeItem, newState) {
			this._setCheckboxState(storeItem, newState, false);
			this._updateChildCheckbox(storeItem, newState);
			this._updateParentCheckbox(storeItem);
		},

		/**
		 * return checkbox of storeItem is checked or unchecked or mixed
		 * in source data
		 */
		getCheckboxState : function(storeItem) {
			var state = {
				checked : undefined,
				mixed : false
			};
			if (storeItem == this.root) {
				// if root's checkbox is not set, set a default value
				// "false";
				if (typeof (storeItem.checkbox) == "undefined") { //$NON-NLS-1$
					this.root.mixedState = false;
					this.root.checkbox = undefined;
					this._setCheckboxState(storeItem, this.checkboxState, false);
					state.checked = this.checkboxState;
				} else {
					state.checked = this.root.checkbox;
					state.mixed = this.root.mixedState;
				}
			} else {
				// get children's status
				state.checked = this.store.getValue(storeItem, this.checkboxIdent);
				state.mixed = this.store.getValue(storeItem, this.checkboxMState);
				if (state.checked == undefined) {
					// set status by default
					this._setCheckboxState(storeItem, this.checkboxState, false);
					state.checked = this.checkboxState;
				}
			}
			return state;
		},

		/**
		 * overwrite parent's method to create a new Item ,and paste it
		 * on a parent Item
		 */
		newItem : function(args, parent, insertIndex) {
			// summary:
			// Creates a new item. See `dojo.data.api.Write` for details
			// on args.
			// Used in drag & drop when item from external source
			// dropped onto tree.
			// description:
			// Developers will need to override this method if new items
			// get added
			// to parents with multiple children attributes, in order to
			// define which
			// children attribute points to the new item.
			var identity = this._getStoreIdentity();
			var pInfo = null;
			var newItem;
			if (parent !== this.root) {
				pInfo = {
					parent : parent,
					attribute : this.childrenAttrs[0]
				};
			} else {
				this.onNewRootItem(args);
			}
			if (identity && args[identity]) {
				// Create new item in the tree,
				newItem = this.store.newItem(args, pInfo);
				if (newItem && (insertIndex != undefined)) {
					// Move new item to desired position
					this.pasteItem(newItem, parent, parent, false, insertIndex);
				}
			}

		},

		/**
		 * overwrite parent's method paset childItem on parentItem
		 */
		pasteItem : function(childItem, oldParentItem, newParentItem, bCopy, insertIndex) {
			// summary:
			// Move or copy an item from one parent item to another.
			// Used in drag & drop
			// tags:
			// extension
			if (oldParentItem === this.root) {
				if (!bCopy) {
					// It's onLeaveRoot()'s responsibility to modify the
					// item so it no longer matches
					// this.query... thus triggering an
					// onChildrenChange() event to notify the Tree
					// that this element is no longer a child of the
					// root node
					this.onLeaveRoot(childItem);
				}
			}
			this.inherited(arguments, [ childItem, oldParentItem === this.root ? null : oldParentItem,
					newParentItem === this.root ? null : newParentItem, bCopy, insertIndex ]);
			if (newParentItem === this.root) {
				// It's onAddToRoot()'s responsibility to modify the
				// item so it matches
				// this.query... thus triggering an onChildrenChange()
				// event to notify the Tree
				// that this element is now a child of the root node
				this.onAddToRoot(childItem);
			}
			this._updateParentCheckbox(childItem);
		},

		/**
		 * delete one Item
		 */
		deleteItem : function(storeItem) {
			// summary:
			// Delete a store item.
			// storeItem:
			// The store item to be delete.

			return this.store.deleteItem(storeItem);
		},

		/**
		 * get Identity of store
		 */
		_getStoreIdentity : function() {
			// summary:
			// Get the identity (identifier) property of the store, if
			// any.
			// tags:
			// private

			if (this._identity) {
				return this._identity;
			}
			if (this._identity === null) {
				var identities = this.store.getIdentityAttributes();
				if (identities) {
					this._identity = identities[0];
				} else {
					this._identity = undefined;
				}
			}
			return this._identity;
		},

		/**
		 * set the checked status in store model
		 */
		_setCheckboxState : function(storeItem, newState, mixedState) {
			var stateChanged = true;
			if (storeItem != this.root) {
				var currState = this.store.getValue(storeItem, this.checkboxIdent);
				var currMixed = this.store.getValue(storeItem, this.checkboxMState);
				if ((currState != newState || currMixed != mixedState) && (currState !== undefined)) {
					this.store.setValue(storeItem, this.checkboxIdent, newState);
					this.store.setValue(storeItem, this.checkboxMState, mixedState);
				} else {
					stateChanged = false;
				}
			} else {
				if ((this.root.checkbox != newState || this.root.mixedState != mixedState)
						&& (this.root.checkbox !== undefined)) {
					this.root.checkbox = newState;
					this.root.mixedState = mixedState;
				} else {
					stateChanged = false;
				}
			}

			return stateChanged;
		},

		/**
		 * set the children checked or not,same as parent's
		 */
		_updateChildCheckbox : function(parentItem, newState) {
			if (this.mayHaveChildren(parentItem)) {
				this.getChildren(parentItem, dojo.hitch(this, function(children) {
					dojo.forEach(children, function(child) {
						this._setCheckboxState(child, newState, false);
						// iterate to set child's checkbox
						if (this.mayHaveChildren(child)) {
							this._updateChildCheckbox(child, newState);
						}
					}, this);
				}), function(err) {
					console.error(_this, ": updating child checkboxes: ", err);//$NON-NLS-1$
				});
			}
		},

		/**
		 * update parent's checkbox according child's state
		 */
		_updateParentCheckbox : function(storeItem) {
			var parentItem = this._getParentItem(storeItem);
			if (!parentItem) {
				return;
			}
			// to judge if all children's checkbox are checked
			this.getChildren(parentItem, dojo.hitch(this, function(brothers) {
				var hasChecked = false;
				var hasUnchecked = false;
				var isMixed = false;
				dojo.some(brothers, function(brother) {
					brotherState = this.getCheckboxState(brother);
					;
					isMixed = brotherState.mixed;
					switch (brotherState.checked) {
					case true:
						hasChecked = true;
						break;
					case false:
						hasUnchecked = true;
						break;
					}
					// if one child is not checked then return true.
					return isMixed;
				}, this);
				isMixed = !(hasChecked ^ hasUnchecked);
				//
				if (this._setCheckboxState(parentItem, isMixed ? true : hasChecked, isMixed ? true : false)) {
					this._updateParentCheckbox(parentItem);
				}

			}), null);

		},

		/**
		 * get one's parent, for this implements,one child can get only
		 * one parents
		 * 
		 */
		_getParentItem : function(storeItem) {
			var parent = null;
			// root do not have parent
			if (storeItem != this.root) {
				var references = storeItem[this.store._reverseRefMap];
				for (itemId in references) {
					parent = this.store._itemsByIdentity[itemId];
					break;
				}
				if (!parent) {
					parent = this.root;
				}
			}
			// every child should have only one parent
			return parent;
		},

	});
})();