/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

(function() {

	var DEFAULT_SA_MESSAGE_PREFIX = "CRCSA";
	
	dojo.provide("psm.common.utils.messaging");
	dojo.require("psm.common.utils.timedEvent");
	dojo.require("jazz.ui.MessageBox");
	
	dojo.declare("psm.common.utils.messaging",null,{
		constructor: function() {
			jazz.ui.MessageBox.setDefaultPrefix(DEFAULT_SA_MESSAGE_PREFIX);
			this.msgHandler = new jazz.ui.MessageBox();
		},

		/***********************************************************************
		 * Create the Jazz MessageBox in the message area
		 * 
		 * @param  args.page - The html page to find the MessageBoxAP dojoattachpoint in
		 **********************************************************************/
		initialiseMessageArea: function(theAttatchPoint)
		{
			theAttatchPoint.appendChild(this.msgHandler.domNode);
		},


		/***********************************************************************
		 * Show a message in the message area (Jazz MessageBox)
		 * 
		 * @param  args.Summary - The text to display 
		 * @param  args.type - The type of message: (optional)
		 * 					'OK'/'ERROR'/'INFO'/'WARNING' 
		 * 					Default = 'OK'
		 * @param  args.Details - Show additional messages/details under a 'Show more'
		 * 					link (optional)
		 * 					Default = null/none
		 * 					Format required:
		 * 					[ {message: "message 1", code: 'OK'   }, 
		 *      			  {message: "message 2", code: 'ERROR'}, ... ]
		 * @param  args.closable - Allow the user to close the message (optional)
		 * 					Default = true
		 * @param  args.timer - time to show the message for.  (optional) 
		 * 					<0 = Permanent
		 * 					Default = 3500ms
		 **********************************************************************/
		showMessage: function(Summary, type, Details, closable, timer)
		{

			// Check the optional arguments, set default values if not provided
			var Type = type ? type : jazz.ui.MessageBox.STATUS_TYPES.OK; 
			var Closable = closable ? closable : true;
			var Timer = timer ? timer : 3500;

			// Hide any existing message
			this.msgHandler.hideMessage();

			// Show the new message
			this.msgHandler.showMessage({ 
				type: Type,
				summary: Summary,
				details: Details,
				closable: Closable
			});

			// If specified, set a timer to hide the message after a delay
			if (Timer > 0)
			{		
				psm.common.utils.timedEvent.runTimedEvent(function(){
					this.msgHandler.hideMessage();
					CanvasUtils.resizeCanvas();
				}, Timer);
			};
		},
		
		hideMessage:function(){
			this.msgHandler.hideMessage();
		}
	});
})();					